# Generated by Django 3.1.7 on 2021-03-15 11:52

import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

import isc_common.fields.code_field
import isc_common.fields.related


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0084_remove_user_email'),
    ]

    operations = [
        migrations.CreateModel(
            name='User_phones',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeStrictField()),
                ('phone', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='isc_common.phones')),
                ('user', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Администрация клуба',
                'unique_together': {('code', 'user', 'phone')},
            },
        ),
        migrations.CreateModel(
            name='User_e_mails',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeStrictField()),
                ('e_mail', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='isc_common.e_mails')),
                ('user', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Администрация клуба',
                'unique_together': {('code', 'user', 'e_mail')},
            },
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-22 22:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0036_auto_20190806_0651'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='message_user_2_user',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='messages',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='messages_state',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='messages_theme',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='messages_theme_access',
            name='id_old',
        ),
    ]



# Generated by Django 3.1.6 on 2021-03-01 10:40

from django.db import migrations

import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0072_userview'),
    ]

    operations = [
        migrations.AlterField(
            model_name='standard_colors',
            name='color',
            field=isc_common.fields.code_field.ColorField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.5 on 2019-09-05 18:10

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0059_auto_20190905_0519'),
        ('twits', '0008_chat_user_user_theme'),
    ]

    operations = [
        migrations.RenameField(
            model_name='chat_user_user_theme',
            old_name='messages_theme',
            new_name='theme',
        ),
        migrations.AlterUniqueTogether(
            name='chat_user_user_theme',
            unique_together={('chat_user_user', 'theme')},
        ),
    ]



import logging

from bitfield import BitField

from isc_common.auth.managers.user_manager import UserManager
from isc_common.auth.models.abstract_user import AbstractUser
from isc_common.fields.code_field import CodeField
from isc_common.fields.description_field import DescriptionField

logger = logging.getLogger(__name__)


class User(AbstractUser):
    description = DescriptionField(null=True, blank=True)
    color = CodeField(null=True, blank=True)
    props = BitField(flags=(
        ('bot', 'Системный пользователь'),  # 1
    ), default=0, db_index=True)

    @classmethod
    def admin_user(cls):
        admin_user, _ = User.objects.get_or_create(username='admin', defaults=dict(password='admin'))
        return admin_user

    @classmethod
    def unknown(cls):
        res, _ = cls.objects.get_or_create(username="unknown")
        return res

    def __str__(self):
        return f"id: {self.id}, username: {self.username}, full_name: ({self.first_name} {self.middle_name} {self.last_name})"

    objects = UserManager()

    class Meta:
        verbose_name = 'Пользователь'
        swappable = 'AUTH_USER_MODEL'



# Generated by Django 3.1.6 on 2021-02-19 09:16

from django.db import migrations, models

import isc_common.fields.files
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0083_auto_20210211_0716'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messages_files',
            name='attfile',
            field=isc_common.fields.files.FileFieldEx(blank=True, max_length=255, null=True, upload_to='', verbose_name='Файл'),
        ),
        migrations.AlterField(
            model_name='messages_files',
            name='mime_type',
            field=isc_common.fields.name_field.NameField(blank=True, null=True, verbose_name='MIME тип файла файла'),
        ),
        migrations.AlterField(
            model_name='messages_files',
            name='real_name',
            field=models.TextField(blank=True, null=True, verbose_name='Первоначальное имя файла'),
        ),
    ]



from isc_common.auth.models.user import User
from isc_common.auth.models.usergroup import UserGroup
from isc_common.http.DSRequest import DSRequest
from isc_common.http.RPCResponse import RPCResponseConstant


class PermissionRequest( DSRequest ) :
    def __init__( self , request ) :
        from permission.models.user_permission import User_permission
        from permission.models.widgets_trees import Widgets_trees

        DSRequest.__init__( self , request )
        data = self.get_data()
        table = data.get( 'table' , None )
        id = data.get( 'id' , None )
        id_widget = data.get( 'id_widget' , None )

        if table :
            if id is not None and id_widget :
                structure = dict()
                if table == 'widgets_trees' :
                    structure = Widgets_trees.objects.get_or_create( id_widget=id_widget )[ 0 ].structure

                if table == 'user' :
                    user = User.objects.get( id=id )
                    widget = Widgets_trees.objects.get_or_create( id_widget=id_widget )[ 0 ]

                    structure = User_permission.objects.get_or_create( user=user , widget=widget )[ 0 ].permission

                if table == 'user_group' :
                    usergroup = UserGroup.objects.get( id=id )
                    widget = Widgets_trees.objects.get_or_create( id_widget=id_widget )[ 0 ]

                self.response = dict( response=dict( status=RPCResponseConstant.statusSuccess ) , data=structure )
        else :
            raise Exception( f"Check table: {table},  id: {id}" )



# Generated by Django 3.2.6 on 2021-08-08 22:46

import bitfield.models
from django.conf import settings
import django.contrib.postgres.fields
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0102_alter_images_size'),
    ]

    operations = [
        migrations.CreateModel(
            name='Users_old_images',
            fields=[
                ('old_ids', django.contrib.postgres.fields.ArrayField(base_field=models.BigIntegerField(), blank=True, null=True, size=None, unique=True)),
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('props', bitfield.models.BitField((('main_photo', 'main_photo'),), db_index=True, default=0)),
                ('image', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='isc_common.images')),
                ('main_model', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Кросс таблица',
                'unique_together': {('image', 'main_model', 'props')},
            },
        ),
    ]



import logging
import re

from django.db import connection
from isc_common import setAttr
from isc_common.number import ToStr

logger = logging.getLogger(__name__)


def uncapitalize(str):
    return str[0:1].lower() + str[1:]


def dbl_qutes_str(str):
    return f'"{str}"'


def qutes_str(str):
    return f"'{str}'"


def delete_dbl_spaces(value):
    if value is None:
        return value

    if not isinstance(value, str):
        return ToStr(value)

    value = value.replace('\\n', ' ')
    return re.sub('\s+', ' ', value).strip()


def null2blanck(str):
    return '' if str == 'null' else str


def ExecuteStoredProc(storedProcName, parametrs):
    c = connection.cursor()
    try:
        # c.execute("BEGIN")
        res = c.callproc(storedProcName, parametrs)
        results, = c.fetchone()
        # c.execute("COMMIT")
        return results
    except Exception as ex:
        logger.error(ex)
        raise ex
    finally:
        c.close()


def ExecuteStoredProcRows(storedProcName, parametrs):
    c = connection.cursor()
    try:
        # c.execute("BEGIN")
        res = c.callproc(storedProcName, parametrs)
        results = c.fetchall()
        # c.execute("COMMIT")
        return results
    except Exception as ex:
        logger.error(ex)
        raise ex
    finally:
        c.close()


class Common:
    @classmethod
    def get_size_file_str(cls, value):
        if value == 0:
            return ""

        if value > 0 and value <= 1024:
            return f'{value} Байт'

        if value > 1024 and value <= 1024 * 1024:
            return f'{round(value / 1024, 2)} КБайт'

        if value > 1024 * 1024 and value <= 1024 * 1024 * 1024:
            return f'{round(value / 1024 / 1024, 2)} МБайт'

        if value > 1024 * 1024 * 1024:
            return f'{round(value / 1024 / 1024 / 1024, 2)}  ГБайт'
        return value

    @classmethod
    def arraund_error(error):
        str = '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
        return f'\n{str}\n{error}\n{str}'


def get_fields_name_from_model(model, data):
    res = list(filter(lambda x: x != 'id' and data.get(x) is not None, map(lambda x: x.name, model._meta.fields)))
    res.extend(list(map(lambda x: f'{x.name}_id', filter(lambda x: x.is_relation is True and x.name not in res, model._meta.fields))))
    return res


def get_relation_field_name(model):
    res = list(map(lambda x: x.name, filter(lambda x: x.is_relation is True and x.name != 'parent', model._meta.fields)))
    return res


def get_dict_only_model_field(data, model, exclude=[]):
    res = dict()
    fields_name = get_fields_name_from_model(model, data)
    for k, v in data.items():
        if k in fields_name and not k in exclude:
            setAttr(res, k, v)
    return res


def check_image_files(apps, schema_editor):
    from isc_common.models.images import Images
    from tqdm import tqdm
    from isc_common.fields.files import make_REPLACE_IMAGE_PATH
    from django.conf import settings
    from lfl_admin.common.management.commands import update_img_refs
    from isc_common.fields.files import FILE_ex
    import os

    query = Images.objects.exclude().order_by('id')
    pbar = tqdm(total=query.count())
    ssh_files = settings.SSH_CLIENTS.client(settings.FILES_STORE)
    o_ssh_files = settings.SSH_CLIENTS.client( settings.OLD_FILES )

    for image in query:

        image_fullpath = make_REPLACE_IMAGE_PATH(image.real_name)
        deleted = False
        if o_ssh_files.exists(image_fullpath) is False:
            logger.debug(f'Deliting : {image}')
            try:
                if FILE_ex(image=image, ssh_files=ssh_files)[0] is False:
                    image.delete()
                    deleted = True
            except Exception as ex:
                deleted = True
                image.file_name = None
                image.size = 0
                image.attfile = ''
                image.save()
                logger.error(ex)

        if deleted is False:
            exists, size = FILE_ex(image=image, ssh_files=ssh_files)
            if exists is True:
                _, file_name = os.path.split(image_fullpath)
                imagen = Images.objects.exclude(id=image.id).getOptional(file_name=file_name, size=size)

                if imagen is not None:
                    update_img_refs(old_id=imagen.id, new_id=image.id)
                image.file_name = file_name
                image.size = size
                image.save()

            else:
                image.file_name = None
                image.size = 0
                image.attfile = ''
                image.save()

        pbar.update()
    logger.info('Done.')



# Generated by Django 3.1.6 on 2021-02-19 06:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0066_auditmodelex'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='auditmodelex',
            name='edited_by',
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-25 11:03

from django.db import migrations
import isc_common.fields.date_time_field
import isc_common.fields.time_field


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0063_auto_20190825_1024'),
    ]

    operations = [
        migrations.AlterField(
            model_name='calendars',
            name='begindate',
            field=isc_common.fields.date_time_field.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Начало'),
        ),
        migrations.AlterField(
            model_name='calendars',
            name='enddate',
            field=isc_common.fields.date_time_field.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Конец'),
        ),
        migrations.AlterField(
            model_name='holidays',
            name='begindate',
            field=isc_common.fields.date_time_field.DateTimeField(db_index=True, verbose_name='Начало'),
        ),
        migrations.AlterField(
            model_name='holidays',
            name='enddate',
            field=isc_common.fields.date_time_field.DateTimeField(db_index=True, verbose_name='Конец'),
        ),
        migrations.AlterField(
            model_name='shift_day',
            name='begintime',
            field=isc_common.fields.time_field.TimeField(db_index=True, verbose_name='Начало'),
        ),
        migrations.AlterField(
            model_name='shift_day',
            name='endtime',
            field=isc_common.fields.time_field.TimeField(db_index=True, verbose_name='Конец'),
        ),
        migrations.AlterField(
            model_name='shift_day_relax',
            name='begintime',
            field=isc_common.fields.time_field.TimeField(db_index=True, verbose_name='Начало периода'),
        ),
        migrations.AlterField(
            model_name='shift_day_relax',
            name='endtime',
            field=isc_common.fields.time_field.TimeField(db_index=True, verbose_name='Конец периода'),
        ),
    ]



import calendar
import unittest
from datetime import datetime

from clndr.calendar_event import CalendarEvent, CalendarEventLinkedList


class FusionChainTestCase(unittest.TestCase):

    def setUp(self) -> None:
        self.shifts = CalendarEventLinkedList()
        self.relaxes = CalendarEventLinkedList()

    def test_одного_года_2_смены_со_всеми_отдыхами_и_перекурами(self):
        year = 2019
        for month in range(1, 12):
            _, month_range = calendar.monthrange(year, month)
            for day in range(1, month_range):
                weekday = calendar.weekday(year, month, day)

                if weekday < 5:
                    self.shifts.add(CalendarEvent(
                        startDate=datetime(year=year, month=month, day=day, hour=7),
                        endDate=datetime(year=year, month=month, day=day, hour=16),
                        iswork=True
                    ))

                    if month_range < day + 1:
                        if month + 1 > 12:
                            self.shifts.add(CalendarEvent(
                                startDate=datetime(year=year, month=month, day=day, hour=16),
                                endDate=datetime(year=year + 1, month=1, day=1, hour=1),
                                iswork=True
                            ))
                        else:
                            self.shifts.add(CalendarEvent(
                                startDate=datetime(year=year, month=month, day=day, hour=16),
                                endDate=datetime(year=year, month=month + 1, day=1, hour=1),
                                iswork=True
                            ))
                    else:
                        self.shifts.add(CalendarEvent(
                            startDate=datetime(year=year, month=month, day=day, hour=16),
                            endDate=datetime(year=year, month=month, day=day + 1, hour=1),
                            iswork=True
                        ))

                    self.relaxes.add(CalendarEvent(
                        startDate=datetime(year=year, month=month, day=day, hour=9),
                        endDate=datetime(year=year, month=month, day=day, hour=9, minute=15),
                        isrelax=True
                    ))

                    self.relaxes.add(CalendarEvent(
                        startDate=datetime(year=year, month=month, day=day, hour=11),
                        endDate=datetime(year=year, month=month, day=day, hour=12),
                        isrelax=True
                    ))

                    self.relaxes.add(CalendarEvent(
                        startDate=datetime(year=year, month=month, day=day, hour=14),
                        endDate=datetime(year=year, month=month, day=day, hour=14, minute=15),
                        isrelax=True
                    ))

                    self.relaxes.add(CalendarEvent(
                        startDate=datetime(year=year, month=month, day=day, hour=18),
                        endDate=datetime(year=year, month=month, day=day, hour=18, minute=15),
                        isrelax=True
                    ))

                    self.relaxes.add(CalendarEvent(
                        startDate=datetime(year=year, month=month, day=day, hour=20),
                        endDate=datetime(year=year, month=month, day=day, hour=21),
                        isrelax=True
                    ))

                    self.relaxes.add(CalendarEvent(
                        startDate=datetime(year=year, month=month, day=day, hour=23),
                        endDate=datetime(year=year, month=month, day=day, hour=23, minute=15),
                        isrelax=True
                    ))

        self.shifts.fusion(self.relaxes)
        self.shifts.print('shifts')


if __name__ == '__main__':
    unittest.main()



import logging
import os
from os import walk
from pathlib import Path

from django.core.management import BaseCommand
from isc_common.ssh.ssh_client import SSH_Client

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('--username', type=str)
        # parser.add_argument('--message', type=str)

    def handle(self, *args, **options):
        SSH_HOST = '192.168.0.158'
        SSH_PORT = 22
        SSH_USER = 'download-image-content'
        SSH_PASSWORD = 'download-image-content'

        c_dumps_path = '/home/download-image-content/images/FILES'
        c_dumps_path = '/home/download-image-content/images'


        SSH_CLIENT = SSH_Client(hostname=SSH_HOST, username=SSH_USER, password=SSH_PASSWORD, port=SSH_PORT)
        res = SSH_CLIENT.exists(f'{c_dumps_path}/.git')
        print(res)



import logging

from django.conf import settings
from django.contrib.postgres.fields import ArrayField
from django.db import connection
from django.db.models import Model, BooleanField, BigAutoField, DateTimeField, Transform, CharField, TextField, BigIntegerField
from django.utils import timezone
from isc_common import setAttr, delAttr
from isc_common.fields.code_field import CodeField
from isc_common.managers.common_manager import CommonManager, CommonQuerySet
from isc_common.number import GetListFromStringCortege
from transliterate import translit
from transliterate.exceptions import LanguageDetectionError

logger = logging.getLogger(__name__)


class Model_withOldId(Model):
    old_id = BigIntegerField(db_index=True, null=True, blank=True, unique=True)

    class Meta:
        abstract = True


class Model_withOldIdStr(Model):
    old_id = CodeField(unique=True)

    class Meta:
        abstract = True


class Model_withOldIds(Model):
    old_ids = ArrayField(BigIntegerField(), null=True, blank=True, unique=True)

    class Meta:
        abstract = True


class AuditQuerySet(CommonQuerySet):
    def getOptional(self, *args, **kwargs):
        try:
            return self.get(*args, **kwargs)
        except self.model.DoesNotExist:
            return None

    def _check_ed_izm_qty(self, **kwargs):
        if kwargs.get('ed_izm') is not None and kwargs.get('qty') is None or kwargs.get('ed_izm') is None and kwargs.get('qty') is not None:
            raise Exception(f'Единица измерения не может быть указана без количества')

    def __init__(self, model=None, query=None, using=None, hints=None, alive_only=True):
        self.alive_only = alive_only
        super().__init__(model=model, query=query, using=using, hints=hints, alive_only=alive_only)

    def rearrange_parent(self, json):
        # _criteria = json.get('data').get('criteria')
        # if isinstance(_criteria, list):
        #     for criterion in _criteria:
        #         if criterion.get('fieldName') == 'parent_id':
        #             item_id = criterion.get('value')
        #             if (item_id, int):
        #                 criteria_lst = [criterion for criterion in _criteria if criterion.get('fieldName') == 'parent_id' and criterion.get('value') is not None]
        #                 if len(criteria_lst) > 0:
        #                     setAttr(json.get('data'), 'criteria', criteria_lst)
        return json

    def soft_delete(self):
        res = super().update(deleted_at=timezone.now())
        if res > 0:
            return self
        else:
            return None

    def soft_restore(self):
        res = super().update(deleted_at=None)
        return res

    def hard_delete(self):
        res = super().delete()
        return res

    def alive(self):
        return self.filter(deleted_at=None)

    def dead(self):
        return self.exclude(deleted_at=None)

    def update(self, **kwargs):
        if kwargs.get('lastmodified') is None:
            setAttr(kwargs, 'lastmodified', timezone.now())
        return super().update(**kwargs)


class AuditManager(CommonManager):
    @classmethod
    def getRecord(cls, record):
        return dict()

    def __init__(self, *args, **kwargs):
        self.alive_only = kwargs.pop('alive_only', True)
        super().__init__(*args, **kwargs)

    def get_queryset(self):
        if self.alive_only:
            return AuditQuerySet(model=self.model, alive_only=self.alive_only).filter(deleted_at=None)
        return AuditQuerySet(model=self.model, alive_only=self.alive_only)

    def hard_delete(self):
        return self.get_queryset().hard_delete()

    def soft_delete(self):
        return self.get_queryset().soft_delete()

    def soft_restore(self):
        return self.get_queryset().soft_restore()

    def getOptional(self, *args, **kwargs):
        try:
            return self.get(*args, **kwargs)
        except self.model.DoesNotExist:
            return None

    def getOr(self, *args, **kwargs):
        alternative = kwargs.get('alternative')
        try:
            delAttr(kwargs, 'alternative')
            return self.get(*args, **kwargs)
        except self.model.DoesNotExist:
            if not callable(alternative):
                raise Exception('alternative must be callable')
            return alternative()
        except self.model.MultipleObjectsReturned as ex:
            old_ids = kwargs.get('old_ids__overlap')
            old_id = kwargs.get('old_id')
            if old_ids is None and old_id is None:
                if not callable(alternative):
                    raise Exception('alternative must be callable')
                return alternative()

            if len(old_ids) > 1:
                raise ex

            old_id = old_ids[0]

            for item in self.filter(*args, **kwargs):
                if len(item.old_ids) > 1:
                    item.old_ids = list(filter(lambda x: x != old_id, item.old_ids))
                    item.save()

            return self.get(*args, **kwargs)

    def get_record(self, record):
        record = self.get(id=record.id if isinstance(record, Model) else record.get('id'))
        self.getRecord(record=record)


class Manager(CommonManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get_queryset(self):
        return AuditQuerySet(model=self.model)


class AuditModel(Model):
    id = BigAutoField(primary_key=True, verbose_name="Идентификатор")
    deleted_at = DateTimeField(verbose_name="Дата мягкого удаления", null=True, blank=True, db_index=True)
    editing = BooleanField(verbose_name="Возможность редактирования", default=True)
    deliting = BooleanField(verbose_name="Возможность удаления", default=True)
    lastmodified = DateTimeField(verbose_name='Последнее обновление', editable=False, db_index=True, default=timezone.now)

    @classmethod
    def get_urls_datas(cls, record, keyimages, model, model_images, main_model, imports, django_model, django_model_images, code=None, add_params=[]):
        from isc_common.number import DelProps

        if not isinstance(record, dict):
            raise Exception('Record must be dict.')

        record = DelProps(record)
        res = list()
        idx = 0
        for keyimage in keyimages:
            real_name, image_src = cls.get_urls_data(
                add_params=add_params[idx] if len(add_params) != 0 else '',
                code=code,
                django_model=django_model,
                django_model_images=django_model_images,
                id=record.get('id'),
                imports=imports,
                keyimage=keyimage,
                main_model=main_model,
                model=model,
                model_images=model_images,
            )
            idx += 1

            real_name, image_id = GetListFromStringCortege(real_name)
            image_src = f'{settings.IMAGE_CONTENT_PROTOCOL}://{settings.IMAGE_CONTENT_HOST}:{settings.IMAGE_CONTENT_PORT}/{image_src}'
            item = {
                f'{keyimage}_image_id': image_id,
                f'{keyimage}_image_src': image_src,
                f'{keyimage}_real_name': real_name,
            }
            res.append(item)

        for urls_data in res:
            for k, v in urls_data.items():
                setAttr(record, k, v)
        return record

    @classmethod
    def get_urls_data(cls, id, keyimage, model, model_images, main_model, imports, django_model, django_model_images, code=None, add_params=''):
        if code is None:
            code = main_model

        if add_params:
            add_params = f'&{add_params}'

        _imports = ','.join(imports)

        with connection.cursor() as cursor:
            sql_str = f'''select (select (ici.real_name, usi.image_id)
                                        from {model_images} as usi
                                                 join isc_common_images ici on ici.id = usi.image_id
                                                 join isc_common_image_types icit on usi.type_id = icit.id
                                        where usi.main_model_id = cd.id
                                          and icit.code = %s
                                          and icit.keyimage = %s
                                          and usi.deleted_at is null
                                        limit 1)                                                                                                           as real_name,
                                       concat('logic/Imgs/Download/', (
                                           select usi.image_id
                                           from {model_images} as usi
                                                    join isc_common_images ici on ici.id = usi.image_id
                                                    join isc_common_image_types icit on usi.type_id = icit.id
                                           where usi.main_model_id = cd.id
                                             and icit.code = %s
                                             and icit.keyimage = %s
                                             and usi.deleted_at is null
                                           limit 1), '?', 
                                           'code={code}', 
                                           '&', 
                                           'keyimage={keyimage}', 
                                           '&', 
                                           'path={main_model}', 
                                           '&', 
                                           'main_model={main_model}', 
                                           '&', 
                                           'main_model_id=', cd.id, 
                                           '&', 
                                           'imports=', '{_imports}', 
                                           '&', 
                                           'django_model=', '{django_model._meta.object_name}',
                                           '&', 
                                           'django_model_images=', '{django_model_images._meta.object_name}',
                                           '{add_params}'
                                           ) as image_src
                                from {model} as cd
                                where cd.id = %s'''

            cursor.execute(sql_str, [code, keyimage, code, keyimage, id])
            record = cursor.fetchone()
            return record

    @classmethod
    def uncapitalize(cls, str):
        from isc_common.common.functions import uncapitalize
        return uncapitalize(str)

    @classmethod
    def translit(cls, value):
        if isinstance(value, int):
            return str(value)
        try:
            return cls.uncapitalize(translit(value, reversed=True).replace("'", '').replace(' ', '_').replace('.', '_'))
        except LanguageDetectionError:
            return cls.uncapitalize(value.replace("'", '').replace(' ', '_').replace('.', '_'))

    @classmethod
    def dbl_qutes_str(cls, str):
        from isc_common.common.functions import dbl_qutes_str
        return dbl_qutes_str(str)

    @classmethod
    def qutes_str(cls, str):
        from isc_common.common.functions import qutes_str
        return qutes_str(str)

    @property
    def idHidden(self):
        return not self is None

    @property
    def is_deleted(self):
        return self.deleted_at is not None

    class Meta:
        abstract = True

    def soft_delete(self):
        self.deleted_at = timezone.now()
        self.save()
        return self

    def soft_restore(self):
        self.deleted_at = None
        self.save()
        return self

    objects = AuditManager()
    all_objects = AuditManager(alive_only=False)


class Dbl_spacesValue(Transform):
    lookup_name = 'delete_dbl_spaces'
    function = 'delete_dbl_spaces'


CharField.register_lookup(Dbl_spacesValue)
TextField.register_lookup(Dbl_spacesValue)


class Trim_Value(Transform):
    lookup_name = 'trim'
    function = 'trim'


CharField.register_lookup(Trim_Value)
TextField.register_lookup(Trim_Value)


class Upper_Value(Transform):
    lookup_name = 'upper'
    function = 'upper'


CharField.register_lookup(Upper_Value)
TextField.register_lookup(Upper_Value)



from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from react.models.fragments import Fragments
from react.models.fragments_view import Fragments_view, Fragments_viewManager


@JsonResponseWithException()
def Fragments_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Fragments_view.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Fragments_viewManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)

@JsonResponseWithException()
def Fragments_Add(request):
    return JsonResponse(DSResponseAdd(data=Fragments.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)

@JsonResponseWithException()
def Fragments_Add_4_Page(request):
    return JsonResponse(DSResponseAdd(data=Fragments.objects.create4PageFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Fragments_Update(request):
    return JsonResponse(DSResponseUpdate(data=Fragments.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Fragments_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Fragments.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)

@JsonResponseWithException()
def Fragments_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Fragments.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Fragments_Info(request):
    return JsonResponse(DSResponse(request=request, data=Fragments_view.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Fragments_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Fragments.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.3 on 2019-07-06 10:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0010_auto_20190629_2304'),
    ]

    operations = [
        migrations.AddField(
            model_name='documents_param_1c',
            name='value_uuid',
            field=models.UUIDField(blank=True, db_index=True, null=True),
        ),
        migrations.AlterField(
            model_name='documents_param_1c',
            name='value',
            field=models.TextField(blank=True, db_index=True, null=True),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-29 23:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0009_auto_20190616_0450'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='documents_param_cross_1c',
            unique_together={('document', 'param')},
        ),
    ]



from django.urls import path

from reports.views import jasper_reports

urlpatterns = [

    path('Jasper_reports/Fetch/', jasper_reports.Jasper_reports_Fetch),
    path('Jasper_reports/Add', jasper_reports.Jasper_reports_Add),
    path('Jasper_reports/Update', jasper_reports.Jasper_reports_Update),
    path('Jasper_reports/Remove', jasper_reports.Jasper_reports_Remove),
    path('Jasper_reports/Lookup/', jasper_reports.Jasper_reports_Lookup),
    path('Jasper_reports/Info/', jasper_reports.Jasper_reports_Info),
    path('Jasper_reports/Copy', jasper_reports.Jasper_reports_Copy),

]



import logging

from bitfield import BitField
from django.db import transaction
from django.forms import model_to_dict

from isc_common import setAttr
from isc_common.auth.models.user import User
from isc_common.fields.related import ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditQuerySet, AuditManager, AuditModel
from isc_common.number import DelProps
from twits.models.chats import Chats, ChatsManager

logger = logging.getLogger(__name__)


class Chat_usersQuerySet(AuditQuerySet):

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Chat_usersManager(AuditManager):

    @classmethod
    def getRecord(cls, record):
        res = {
            'id': record.id,
            'chat_id': record.chat.id,
            'user_id': record.user.id,
            'user__username': record.user.username,
            'user__first_name': record.user.first_name,
            'user__last_name': record.user.last_name,
            'user__email': record.user.email,
            'user__middle_name': record.user.middle_name,
            'props': record.props,
            'compulsory_reading': record.props.compulsory_reading,
            'relevant': record.props.relevant,
            'readonly': record.props.readonly,
        }
        return DelProps(res)

    def get_queryset(self):
        return Chat_usersQuerySet(self.model, using=self._db)

    def createFromRequest(self, request, removed=None):
        request = DSRequest(request=request)
        data = request.get_data()
        user_ids = data.get('user_ids', None)
        context_ids = data.get('context_ids', None)
        _data = []

        with transaction.atomic():
            if user_ids and context_ids:
                if not isinstance(user_ids, list):
                    user_ids = [user_ids]

                if not isinstance(context_ids, list):
                    context_ids = [context_ids]

                for user_id in user_ids:
                    for chat_id in context_ids:
                        res = super().create(user_id=user_id, chat_id=chat_id)

                        ChatsManager.refresh_chat_menu(user=res.user, logger=logger)
                        _data.append(model_to_dict(res))

            return data

    def updateFromRequest(self, request, removed=None, function=None):
        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()

        with transaction.atomic():
            props = data.get('props')
            if data.get('compulsory_reading') is True:
                props |= Chat_users.props.compulsory_reading
            else:
                props &= ~Chat_users.props.compulsory_reading

            if data.get('relevant') is True:
                props |= Chat_users.props.relevant
            else:
                props &= ~Chat_users.props.relevant

            if data.get('readonly') is True:
                props |= Chat_users.props.readonly
            else:
                props &= ~Chat_users.props.readonly

            _data = dict()
            setAttr(_data, 'props', props)

            res, _ = super().update_or_create(id=data.get('id'), defaults=_data)
            ChatsManager.refresh_chat_menu(user=res.user, logger=logger)

            return data

    def deleteFromRequest(self, request, removed=None, ):
        request = DSRequest(request=request)
        res = 0
        tuple_ids = request.get_tuple_ids()
        with transaction.atomic():
            for id, mode in tuple_ids:
                if mode == 'hide':
                    super().filter(id=id).soft_delete()
                elif mode == 'visible':
                    super().filter(id=id).soft_restore()
                else:
                    chat_users = super().get(id=id)
                    chat_users.delete()
                    qty = 1

                    ChatsManager.refresh_chat_menu(user=chat_users.user, logger=logger)
                res += qty

            return res

    @classmethod
    def props(cls):
        return BitField(flags=(
            ('compulsory_reading', 'Обязательное прочтение'),  # 1
            ('relevant', 'Актуальность'),  # 2
            ('readonly', 'Только чтение'),  # 4
        ), default=0, db_index=True)


class Chat_users(AuditModel):
    user = ForeignKeyCascade(User)
    chat = ForeignKeyCascade(Chats)
    props = Chat_usersManager.props()

    objects = Chat_usersManager()

    def __str__(self):
        return f"id: {self.id}, user: [{self.user}] chat: [{self.chat}]"

    class Meta:
        verbose_name = 'Участники чатов'
        unique_together = (('user', 'chat'),)



# Generated by Django 3.2.6 on 2021-08-10 15:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('react', '0014_pages_type'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='fragment_params',
            name='type',
        ),
        migrations.DeleteModel(
            name='Fragment_param_types',
        ),
    ]



import json
import logging
import os
from tempfile import TemporaryFile

from crypto.models.upload_file import DSResponse_UploadFile
from django.conf import settings
from django.core.files import File
from django.db import transaction
from isc_common.common import SFTP , FILES_STORE_UNKNOWN_TYPE
from isc_common.common.UploadItem import UploadItem
from isc_common.dropzone import Dz
from isc_common.fields.files import make_REPLACE_FILE_PATH
from isc_common.http.DSResponse import JsonResponseWithException
from isc_common.models.image_types import Image_types
from isc_common.models.images import Images
from websocket import create_connection

from lfl_admin.common.management.commands import delete_img_refs

logger = logging.getLogger( __name__ )


class Common_UploadImage( DSResponse_UploadFile ) :
    def upload_image( self , request ) :

        self.dictionary = dict( id=request.GET.get( 'id' ) )

        self.dz_dictionary = Dz( request.POST )

        self.host = request.GET.get( 'host' )
        self.port = request.GET.get( 'port' )
        self.image_type_name = request.GET.get( 'image_type_name' )
        self.keyImage = request.GET.get( 'keyImage' )

        if self.image_type_name is None :
            raise Exception( 'Не задан параметр image_type_name.' )
        if not self.port :
            self.port = 80
        self.channel = request.GET.get( 'ws_channel' )
        self.file = request.FILES.get( 'upload_attatch' )

        self.dictionary.update( dict(
            real_file_name=self.file.name ,
            stored_file_name=self.dz_dictionary.dzuuid ,
            file_size=int( self.dz_dictionary.dztotalfilesize ) ,
            file_mime_type=self.file.content_type )
        )

        item = UploadItem( dictionary=self.dictionary )

        def load_str( pers ) :
            return f'Загружено: {pers} %'

        image_type , _ = Image_types.objects.get_or_create( code=self.image_type_name , keyimage=self.keyImage , defaults=dict( name=self.image_type_name ) )

        res = Images.objects.getOptional(
            size=item.file_size ,
            real_name=item.real_file_name ,
        )

        if res is not None :
            attfile = make_REPLACE_FILE_PATH( str( res.attfile ) )

            if isinstance( settings.FILES_STORE , str ) :
                ex = os.path.exists( attfile )
                if ex is False :
                    delete_img_refs( old_id=res.id )
                    res = None
            else :
                ssh_clients = settings.SSH_CLIENTS.client( settings.FILES_STORE )
                ex = ssh_clients.exists( attfile )
                if ex is False :
                    delete_img_refs( old_id=res.id )
                    res = None

        if res is None :
            with TemporaryFile() as src :
                src.seek( int( self.dz_dictionary.dzchunkbyteoffset ) )
                src.write( self.file.read() )

                pers = round( (int( self.dz_dictionary.dzchunkindex ) * 100) / int( self.dz_dictionary.dztotalchunkcount ) , 2 )
                if self.last_chunk :
                    pers = 100

                logger.debug( f'{load_str( pers )}, шаг : {int( self.dz_dictionary.dzchunkindex ) + 1} из {self.dz_dictionary.dztotalchunkcount}' )

                if self.last_chunk :
                    logger.debug( f'Загружен файл: {item.real_file_name}.' )
                    logger.debug( f'Запись временного файла.' )

                    with open( item.full_path , 'w+b' ) as destination :
                        src.seek( 0 )
                        destination.write( src.read() )
                        logger.debug( f'Запись временного файла выполнена.' )

                    logger.debug( f'Запись: {item.full_path}.' )
                    with open( item.full_path , 'rb' ) as src :
                        src.seek( 0 )
                        fileObj = File( src )

                        if isinstance( settings.FILES_STORE , str ) :
                            file_store = os.path.abspath( settings.FILES_STORE )
                        elif isinstance( settings.FILES_STORE , dict ) and settings.FILES_STORE.get( 'mode' ) == SFTP :
                            file_store = settings.FILES_STORE.get( 'PATH' )
                        else :
                            raise Exception( FILES_STORE_UNKNOWN_TYPE )

                        res = Images.objects.create(
                            attfile=fileObj ,
                            file_store=file_store ,
                            format=item.file_format ,
                            mime_type=item.file_mime_type ,
                            size=item.file_size ,
                            real_name=item.real_file_name ,
                        )

                        ws = create_connection( f"ws://{self.host}:{self.port}/ws/{self.channel}/" )
                        ws.send( json.dumps( dict( id=item.id , item_id=res.id , type="uploaded" ) ) )
                        ws.close()

                    logger.debug( f'Запись: {item.real_file_name} ({fileObj.name}) завершена.' )

                    self.remove( item.full_path )
                    logger.debug( f'Удаление: {item.full_path} завершено.' )

        return res , image_type , item.id

    @property
    def last_chunk( self ) :
        res = int( self.dz_dictionary.dztotalchunkcount ) == int( self.dz_dictionary.dzchunkindex ) + 1
        return res


@JsonResponseWithException()
class DSResponse_UploadImage( Common_UploadImage ) :
    def __init__( self , request ) :
        self.upload_image( request=request )


@JsonResponseWithException()
class DSResponse_CommonUploadImage( Common_UploadImage ) :
    def __init__( self , request , model , image_model , field_main_model='' ) :
        with transaction.atomic() :
            file = request.FILES.get( 'upload_attatch' )
            if file is not None :
                image , type , id = self.upload_image( request=request )

                main_model = eval( f"model.objects.get( id={id} ){f'.{field_main_model}' if field_main_model else ''}" )
                if main_model is None :
                    raise Exception( f'Для field_main_model {field_main_model} не найдено main_model' )

                main_model_image = image_model.objects.getOptional( main_model=main_model , image=image , type=type , deleted_at__isnull=False )

                if main_model_image is not None :
                    main_model_image.soft_restore()
                    main_model_image.save()
                else :
                    main_model_image = image_model.objects.create( main_model=main_model , type=type , image=image )

                image_model.objects.filter( main_model=main_model , type=type , deleted_at__isnull=True ).exclude( id=main_model_image.id ).soft_delete()

                logger.debug( f'Created: {main_model_image}' )



import logging

from django.db import transaction
from django.db.models import CheckConstraint , F , Q
from isc_common import delAttr , setAttr
from isc_common.auth.models.user import User
from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_tree_grid_manager import CommonTreeGridManager
from isc_common.models.base_ref import BaseRefHierarcy

logger = logging.getLogger( __name__ )


class Messages_theme_Manager( CommonTreeGridManager ) :
    @classmethod
    def getRecord( cls , record ) :
        res = {
            "id" : record.id ,
            "code" : record.code ,
            "name" : record.name ,
            # "full_name": record.full_name,
            "description" : record.description ,
            "parent_id" : record.parent_id ,
            "lastmodified" : record.lastmodified ,
            "editing" : record.editing ,
            "deliting" : record.deliting ,
        }
        return res

    def createFromRequest( self , request ) :
        from tracker.models.messages_theme_users_access import MessagesThemeUsersAccess

        _request = DSRequest( request=request )
        user_id = _request.user.id
        data = _request.get_data()
        _data = data.copy()
        delAttr( _data , 'id' )
        delAttr( _data , 'isFolder' )
        delAttr( _data , 'full_name' )
        setAttr( _data , 'creator_id' , user_id )
        res = super().create( **_data )

        MessagesThemeUsersAccess.objects.get_or_create( theme=res , user_id=user_id )
        return data

    def updateFromRequest( self , request ) :
        from tracker.models.messages import Messages

        request = DSRequest( request=request )
        data = request.get_data()
        mode = data.get( 'mode' )
        dropRecords = data.get( 'dropRecords' )
        targetRecord = data.get( 'targetRecord' )

        if mode == 'move' :
            res = 0
            for dropRecord in dropRecords :
                res += Messages.objects.filter( id=dropRecord ).update( theme_id=targetRecord )
            return res

        _list = [ value for key , value in data.items() ]
        is_list_of_dict = len( [ a for a in _list if isinstance( a , dict ) ] ) > 0
        if is_list_of_dict :
            with transaction.atomic() :
                for _data in _list :
                    if isinstance( _data , dict ) :
                        delAttr( _data , 'isFolder' )
                        delAttr( _data , 'full_name' )
                        delAttr( _data , 'children' )
                        res = super().filter( id=_data.get( 'id' ) ).update( **_data )
        else :
            delAttr( data , 'isFolder' )
            delAttr( data , 'full_name' )
            delAttr( data , 'children' )
            res = super().filter( id=data.get( 'id' ) ).update( **data )
        return res


class Messages_theme( BaseRefHierarcy ) :
    code = CodeField( unique=True )
    creator = ForeignKeyCascade( User , null=True , blank=True )

    @classmethod
    def auto_from_error( cls ) :
        return Messages_theme.objects.get_or_create(
            code='auto_from_error' ,
            defaults=dict(
                name='Автоматически занесенные из сообщений об ошибках.' ,
                creator=User.admin_user() ,
                editing=False ,
                deliting=False ,
                description='Тема создана автоматически' ) )[ 0 ]

    @classmethod
    def chats_theme( cls ) :
        return Messages_theme.objects.get_or_create(
            code='chats' ,
            defaults=dict(
                name='Сообщения чатов' ,
                creator=User.admin_user() ,
                editing=False ,
                deliting=False ,
                description='Тема создана автоматически' ) )[ 0 ]

    @classmethod
    def group_chats_theme( cls ) :
        return Messages_theme.objects.get_or_create(
            code='group_chats' ,
            defaults=dict(
                name='Групповые чаты' ,
                creator=User.admin_user() ,
                editing=False ,
                deliting=False ,
                parent=Messages_theme.chats_theme() ,
                description='Тема создана автоматически' ) )[ 0 ]

    @classmethod
    def tetatet_chats_theme( cls ) :
        return Messages_theme.objects.get_or_create(
            code='tetatet_chats' ,
            defaults=dict(
                name='Чаты личной переписки' ,
                creator=User.admin_user() ,
                editing=False ,
                deliting=False ,
                parent=Messages_theme.chats_theme() ,
                description='Тема создана автоматически' ) )[ 0 ]

    @classmethod
    def system_chats_theme( cls ) :
        return Messages_theme.objects.get_or_create(
            code='system_chats' ,
            defaults=dict(
                name='Чаты системных сообщений' ,
                creator=User.admin_user() ,
                editing=False ,
                deliting=False ,
                parent=Messages_theme.chats_theme() ,
                description='Тема создана автоматически' ) )[ 0 ]

    def __str__( self ) :
        return f"id: {self.code}, id: {self.code}, name: {self.name}, description: {self.description}"

    objects = Messages_theme_Manager()

    class Meta :
        verbose_name = 'Темы задач'
        constraints = [
            CheckConstraint( check=~Q( id=F( 'parent_id' ) ) , name=f'c_Messages_theme' ) ,
        ]



import logging

from django.db.models import PositiveIntegerField, TextField, BooleanField

from isc_common.fields.code_field import CodeStrictField
from isc_common.models.base_ref import BaseRef, BaseRefManager, BaseRefQuerySet

logger = logging.getLogger(__name__)


class Type_paramQuerySet(BaseRefQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Type_paramManager(BaseRefManager):

    @classmethod
    def getRecord(cls, record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'description': record.description,
            'type': record.type,
            'length': record.length,
            # 'precision': record.precision,
            # 'exponent': record.exponent,
        }
        return res

    def get_queryset(self):
        return Type_paramQuerySet(self.model, using=self._db)


class Type_param(BaseRef):
    @classmethod
    def string(cls):
        return 'Строковый'

    @classmethod
    def date(cls):
        return 'Дата без времени'

    @classmethod
    def date_time(cls):
        return 'Дата со временем'

    @classmethod
    def integer(cls):
        return 'Целое число'

    @classmethod
    def float(cls):
        return 'Дробное число'

    @classmethod
    def select(cls):
        return 'Выбор из списка'

    type = CodeStrictField(default='Строка')
    value_map = TextField(null=True, blank=True)
    length = PositiveIntegerField(default=255, null=True, blank=True)
    required = BooleanField(default=False)

    objects = Type_paramManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Тип параметр'



# Generated by Django 3.2.6 on 2021-08-23 18:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0119_alter_image_types_code'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='image_types',
            unique_together={('code', 'keyimage')},
        ),
    ]



import logging

from django.forms import model_to_dict
from isc_common import delAttr
from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditManager
from isc_common.models.base_ref import BaseRef

logger = logging.getLogger( __name__ )


class GroupManager( AuditManager ) :
    @classmethod
    def getRecord( cls , record ) :
        res = {
            "id" : record.id ,
            "code" : record.code ,
            "name" : record.name ,
            "description" : record.description ,
            "parent_id" : record.parent_id ,
            "lastmodified" : record.lastmodified ,
            "editing" : record.editing ,
            "deliting" : record.deliting ,
        }
        return res

    def createFromRequest( self , request ) :
        from permission.models.widgets_trees import Widgets_trees
        from permission.models.usergroup_permission import Usergroup_permission

        request = DSRequest( request=request )
        data = request.get_data()
        _data = data.copy()
        delAttr( _data , 'dataSource' )
        delAttr( _data , 'operationType' )
        delAttr( _data , 'textMatchStyle' )
        delAttr( _data , 'form' )
        delAttr( _data , 'id' )

        res = super().create( **_data )
        for widget in Widgets_trees.objects.all() :
            Usergroup_permission.objects.get_or_create( usergroup=res , widget=widget )

        res = model_to_dict( res )
        data.update( res )
        return data


class UserGroup( BaseRef ) :
    code = CodeField( unique=True )
    parent = ForeignKeyProtect( 'self' , null=True , blank=True )

    @property
    def is_admin( self ) :
        return self.code.lower() == 'administrators' or self.code.lower() == 'admin'

    @property
    def is_develop( self ) :
        return self.code == 'developers'

    def __str__( self ) :
        return f"{self.code}"

    objects = GroupManager()

    class Meta :
        verbose_name = 'группа'



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from lfl_admin.common.models.edited_by import Edited_by, Edited_byManager


@JsonResponseWithException()
def Edited_by_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Edited_by.objects.
                select_related().
                get_range_rows1(
                request=request,
                function=Edited_byManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Edited_by_Add(request):
    return JsonResponse(DSResponseAdd(data=Edited_by.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Edited_by_Update(request):
    return JsonResponse(DSResponseUpdate(data=Edited_by.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Edited_by_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Edited_by.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Edited_by_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Edited_by.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Edited_by_Info(request):
    return JsonResponse(DSResponse(request=request, data=Edited_by.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Edited_by_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Edited_by.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



import logging
import os
from logging import Logger
from threading import Lock

from django.conf import settings

logger1 = logging.getLogger(__name__)


def dictinct_list(l, _sorted=False, key=None):
    if l is None:
        return None

    ds = []
    if isinstance(l, list):
        ds = list(set(l.copy()))

    if isinstance(l, map):
        ds = list(set(list(l).copy()))

    if _sorted:
        return sorted(ds, key=lambda x: x[key])
    return ds


def setAttr(o, name, value):
    if not isinstance(o, dict):
        return o

    o[name] = value
    return o


def delAttr(o, name):
    if not isinstance(o, dict):
        return o

    if name in o:
        del o[name]
        return True
    else:
        return False


def delAttr1(o, name):
    if not isinstance(o, dict):
        return o

    _o = o.copy()
    if name in _o:
        del _o[name]
        return _o
    else:
        return _o


def isEmptyDict(dictionary):
    for element in dictionary:
        if element:
            return True
        return False


def delete_drive_leter(path):
    altsep = os.path.altsep
    if altsep is None:
        altsep = '\\'

    path = path.replace(os.path.sep, altsep)
    if path.find(':') != -1:
        path = (path.split(':')[1]).replace(os.path.sep, altsep)
        path = ''.join(path)

    if path.startswith(altsep):
        return path[1:]
    return path


def get_drive_leter(path):
    path = path.replace(os.path.sep, os.path.altsep)
    if path.find(':') != -1:
        return f"{(path.split(':')[0])}:"
    return None


def replace_alt_set(path):
    from isc_common.fields.files import FieldFileEx
    if isinstance(path, FieldFileEx):
        path = path.path
        return path.replace(os.altsep, os.sep).replace(f'{os.sep}{os.sep}', os.sep)
    else:
        return path.replace(os.altsep, os.sep).replace(f'{os.sep}{os.sep}', os.sep)


def replace_sep(path):
    return path.replace(os.sep, os.altsep)


def del_last_not_digit(str):
    res = ''
    flag = False
    for ch in reversed(str):
        if flag or ch.isdigit():
            res += ch
            flag = True

    return res[::-1]


def str_to_bool(s):
    if s in ['True', 'true']:
        return True
    elif s in ['False', 'false']:
        return False
    else:
        raise ValueError(f'{s} is not a good boolean string')


def bool_to_jsBool(value):
    if value is True:
        return "true"
    return "false"


class StackElementNotExist(Exception):
    def __init__(self, *args, **kwargs):  # real signature unknown
        if kwargs.get('message'):
            super().__init__(kwargs.get('message'))
        else:
            super().__init__('Соответствие не найдено.')


class MultipleStackElement(Exception):
    def __init__(self, *args, **kwargs):  # real signature unknown
        super().__init__('Неоднозначный выбор.')


def cleanProgresses(id_progresses=None, exclude_processes=None):
    from isc_common.models.deleted_progresses import Deleted_progresses
    from isc_common.models.progresses import Progresses

    key = 'cleanProgresses'
    if settings.LOCKS.locked(key):
        return

    settings.LOCKS.acquire(key)
    try:
        if isinstance(id_progresses, str):
            _id_progresses = []
            _id_progresses.append(id_progresses)
            id_progresses = _id_progresses

        if isinstance(exclude_processes, str):
            _exclude_processes = []
            _exclude_processes.append(exclude_processes)
            exclude_processes = _exclude_processes

        if isinstance(id_progresses, list):
            for process in Progresses.objects.filter():
                for p in id_progresses:
                    if str(process.id_progress).find(p) != -1:
                        if isinstance(exclude_processes, list):
                            exc = False
                            for ep in exclude_processes:
                                if str(process.id_progress).find(ep) != -1:
                                    exc = True
                                    break
                            if not exc:
                                Progresses.objects.filter(id=process.id).delete()
                        else:
                            Progresses.objects.filter(id=process.id).delete()

            for process in Deleted_progresses.objects.filter():
                for p in id_progresses:
                    if str(process.id_progress).find(p) != -1:
                        if isinstance(exclude_processes, list):
                            exc = False
                            for ep in exclude_processes:
                                if str(process.id_progress).find(ep) != -1:
                                    exc = True
                                    break
                            if not exc:
                                Deleted_progresses.objects.filter(id=process.id).delete()
                        else:
                            Deleted_progresses.objects.filter(id=process.id).delete()
        else:
            if isinstance(exclude_processes, list):
                for process in Progresses.objects.filter():
                    if isinstance(exclude_processes, list):
                        exc = False
                        for ep in exclude_processes:
                            if str(process.id_progress).find(ep) != -1:
                                exc = True
                                break
                        if not exc:
                            Progresses.objects.filter(id=process.id).delete()
                    else:
                        Progresses.objects.filter(id=process.id).delete()

                for process in Deleted_progresses.objects.filter():
                    if isinstance(exclude_processes, list):
                        exc = False
                        for ep in exclude_processes:
                            if str(process.id_progress).find(ep) != -1:
                                exc = True
                                break
                        if not exc:
                            Deleted_progresses.objects.filter(id=process.id).delete()
                    else:
                        Deleted_progresses.objects.filter(id=process.id).delete()
            else:
                Progresses.objects.filter().delete()
                Deleted_progresses.objects.filter().delete()
        settings.LOCKS.release(key)
    except:
        settings.LOCKS.release(key)


class NotImplement(Exception):
    def __init__(self, text=''):
        super().__init__(f'{text} Нет реализации !!!!!!')


class Stack:
    stack = []

    def __iter__(self):
        return self.stack.__iter__()

    def __init__(self, stack=[]):
        self.stack = stack.copy()

    def top(self, index=1):
        if len(self.stack) < index:
            return None
        return self.stack[len(self.stack) - index]

    def clear(self):
        self.stack = []

    def extend(self, arr):
        self.stack.extend(arr)

    def pop(self):
        if len(self.stack) < 1:
            return None
        return self.stack.pop()

    def reverse(self):
        return Stack(stack=list(reversed(self.stack)))

    def distinct(self):
        return list(set(self.stack))

    def extend(self, lst):
        if isinstance(lst, list):
            for itm in lst:
                self.push(itm)

    def push(self, item, exists_function=None, logger=None):
        if callable(exists_function):
            if exists_function(item, self.stack) == False:
                self.stack.append(item)
                if isinstance(logger, Logger):
                    logger.debug(f'self.stack.append: {item}')
                return True
            else:
                if isinstance(logger, Logger):
                    logger.debug(f'self.stack.not append: {item}')
                return False
        else:
            self.stack.append(item)
            if isinstance(logger, Logger):
                logger.debug(f'self.stack.append: {item}')
            return True

        if isinstance(logger, Logger):
            logger.debug(f'size stack: {len(self.stack)}')
        return True

    def size(self):
        return len(self.stack)

    def copy(self):
        return Stack(self.stack)

    def find(self, function):
        return [item for item in self.stack if function(item)].copy()

    def exists(self, function):
        return len(self.find(function=function)) > 0

    def print_all(self):
        if self.size() > 0:
            logger1.warning('====================================dump===========================================================================')
            for item in self.stack:
                logger1.warning(item)
            logger1.warning('====================================end dump=======================================================================')

    def find_one(self, function):
        res = self.find(function=function).copy()
        if len(res) == 0:
            # self.print_all()
            raise StackElementNotExist()
        elif len(res) > 1:
            raise MultipleStackElement()

        return res[0]

    def __str__(self):
        return "[" + ",".join([str(item) for item in self.stack]) + "]"


class StackLocs(Stack):
    def acquire(self, id):
        try:
            id, lock = self.find_one(lambda x: x[0] == id)
        except StackElementNotExist:
            lock = Lock()
            self.push((id, lock))
        lock.acquire()
        logger1.debug(f'lock.acquire() ID: {id}')

    def locked(self, id):
        try:
            id, lock = self.find_one(lambda x: x[0] == id)
            return lock.locked()
        except StackElementNotExist:
            return False

    def release(self, id):
        try:
            id, lock = self.find_one(lambda x: x[0] == id)
            if lock.locked():
                lock.release()
                logger1.debug(f'lock.release() ID: {id}')
        except StackElementNotExist:
            pass


class StackWithId(Stack):
    def __init__(self, id='id', raise_on_dublicate=True):
        self.id = id
        self.raise_on_dublicate = raise_on_dublicate
        super().__init__()

    def push(self, item, logger=None):
        if item is None:
            return

        if not isinstance(item, dict):
            raise Exception(f'{item} must be dict')

        if not self.exists(item.get(self.id)):
            self.stack.append(item)
            if isinstance(logger, Logger):
                logger.debug(f'self.stack.append: {item}')
                logger.debug(f'self.stack.size: {self.size()}')

            if isinstance(logger, Logger):
                logger.debug(f'size stack: {len(self.stack)}')

            return True
        else:
            if self.raise_on_dublicate is True:
                raise Exception(f'{item} will be doublicate')
            else:
                if isinstance(logger, Logger):
                    logger.warning(f'{item} will be doublicate')

    def exists(self, id):
        return len(self.find(id)) > 0

    def find(self, id):
        if not isinstance(id, int):
            raise Exception(f'{id} must be int')
        return list(filter(lambda x: x.get(self.id) == id, self.stack))

    def find_one(self, id):
        if not isinstance(id, int):
            raise Exception(f'{id} must be int')

        res = self.find(id=id).copy()
        if len(res) == 0:
            raise StackElementNotExist()
        elif len(res) > 1:
            raise MultipleStackElement()

        return res[0]


def get_list_nums_key_dict(data):
    res = []
    if isinstance(data, dict):
        if data.get('0'):
            idx = 0
            while True:
                _data = data.get(str(idx))
                if isinstance(_data, dict):
                    res.append(_data)
                else:
                    break
                idx += 1
        else:
            res.append(data)
    return res


class Wrapper:
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v() if callable(v) else v)

    @property
    def dict(self):
        return self.__dict__

    def __str__(self):
        return str(self.dict)


user_name_anonim = "anonim"


def delNonePropFromDict(in_dict):
    if not isinstance(in_dict, dict):
        raise Exception(f'{in_dict} must be dict')

    res = dict()
    for k, v in in_dict.items():
        if v is not None:
            setAttr(res, k, v)
    return res


def delZeroPropFromDict(in_dict):
    if not isinstance(in_dict, dict):
        raise Exception(f'{in_dict} must be dict')

    res = dict()
    for k, v in in_dict.items():
        if v != 0:
            setAttr(res, k, v)
    return res



import logging

from bitfield import BitField
from django.conf import settings
from django.db import transaction
from django.db.models import TextField, DecimalField, ProtectedError, SmallIntegerField
from isc_common import delAttr, setAttr
from isc_common.common.functions import get_dict_only_model_field
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import BaseRef, BaseRefManager, BaseRefQuerySet
from isc_common.models.images import Images
from react.models.fragment_param_types import Fragment_param_types
from react.models.fragments import Fragments

logger = logging.getLogger(__name__)


class Fragment_paramsQuerySet(BaseRefQuerySet):
    pass


class Fragment_paramsManager(BaseRefManager):

    def updateFromRequest(self, request, propsArr = None):
        request = DSRequest(request=request)
        data = request.get_data()

        data = self.check_data_for_multi_select(data=data)

        cloned_data = data.copy()
        props = self.get_prp(data=data, propsArr=propsArr)
        if props is not None:
            setAttr(cloned_data, 'props', props)

        id = cloned_data.get('id')
        parent = cloned_data.get('parent')

        with transaction.atomic():
            image = super().get(id=id).image
            setAttr(cloned_data, 'image', image)
            delAttr(cloned_data, 'image_id')

            if cloned_data.get('image_real_name') == 'DELETED':
                ex = settings.SSH_CLIENTS.client(settings.FILES_STORE).exists(str(image.attfile))
                if ex is False:
                    try:
                        image.delete()
                    except ProtectedError:
                        pass

                setAttr(cloned_data, 'image_id', None)
                setAttr(cloned_data, 'image', None)

            if isinstance(parent, int):
                delAttr(cloned_data, 'parent')
                setAttr(cloned_data, 'parent_id', parent)
            res = super().filter(id=id).update(**get_dict_only_model_field(data=cloned_data, model=self.model, exclude=['id']))
            logger.debug(f'res: {res}')

        return data

    @classmethod
    def props(cls):
        return BitField(flags=(
            ('use_generator', 'Используется генератором'),  # 1
        ), default=0, db_index=True)

    @classmethod
    def getRecord(cls, record ) :
        res = {
            'code': record.code,
            'deliting': record.deliting,
            'description': record.description,
            'editing': record.editing,
            'fragment_id': record.fragment.id,
            'id': record.id,
            'name': record.name,
            'num': record.num,
            'path_file': record.path_file,
            'type': record.type.id,
            'type__name': record.type.name,
            'type_id': record.type.id,
            'value_dec': record.value_dec,
            'value_text': record.value_text,
        }
        return res

    def get_queryset(self):
        return Fragment_paramsQuerySet(self.model, using=self._db)


class Fragment_params(BaseRef):
    fragment = ForeignKeyProtect(Fragments)
    image = ForeignKeyProtect(Images, null=True, blank=True)
    num = SmallIntegerField(null=True, blank=True)
    path_file = TextField(null=True, blank=True)
    props = Fragment_paramsManager.props()
    type = ForeignKeyProtect(Fragment_param_types)
    value_dec = DecimalField(decimal_places=4, max_digits=19, null=True, blank=True)
    value_text = TextField(null=True, blank=True)

    objects = Fragment_paramsManager()

    def __str__(self):
        return f'ID:{self.id}, ' \
               f'code: {self.code}, ' \
               f'name: {self.name}, ' \
               f'description: {self.description}, ' \
               f'num: {self.num}, ' \
               f'path_file: {self.path_file}, ' \
               f'value_dec: {self.value_dec}, ' \
               f'value_text: {self.value_text}, ' \
               f'type: [{self.type}], ' \
               f'fragment: [{self.fragment}]'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Параметры фрагмента'



from __future__ import unicode_literals

import asyncio
import random
import string
import time
from copy import deepcopy

from channels.exceptions import ChannelFull
from channels.layers import BaseChannelLayer


class AsyncInMemoryChannelLayer(BaseChannelLayer):
    """
    In-memory channel layer implementation
    """

    def __init__(
            self,
            expiry=60,
            group_expiry=3600,
            capacity=1000,
            channel_capacity=None,
            **kwargs
    ):
        super().__init__(
            expiry=expiry,
            capacity=capacity,
            channel_capacity=channel_capacity,
            **kwargs
        )
        self.channels = {}
        self.groups = {}
        self.group_expiry = group_expiry

    ### Channel layer API ###

    extensions = ["groups", "flush"]

    async def send(self, channel, message):
        """
        Send a message onto a (general or specific) channel.
        """
        # Typecheck
        assert isinstance(message, dict), "message is not a dict"
        assert self.valid_channel_name(channel), "Channel name not valid"
        # If it's a process-local channel, strip off local part and stick full name in message
        assert "__asgi_channel__" not in message

        queue = self.channels.setdefault(channel, asyncio.Queue())
        # Are we full
        if self.capacity and (queue.qsize() >= self.capacity):
            # raise ChannelFull(channel)
            queue.empty()

        # Add message
        await queue.put((time.time() + self.expiry, deepcopy(message)))

    async def receive(self, channel):
        """
        Receive the first message that arrives on the channel.
        If more than one coroutine waits on the same channel, a random one
        of the waiting coroutines will get the result.
        """
        assert self.valid_channel_name(channel)
        self._clean_expired()

        queue = self.channels.setdefault(channel, asyncio.Queue())

        # Do a plain direct receive
        _, message = await queue.get()

        # Delete if empty
        if queue.empty():
            del self.channels[channel]

        return message

    async def new_channel(self, prefix="specific."):
        """
        Returns a new channel name that can be used by something in our
        process as a specific channel.
        """
        return "%s.inmemory!%s" % (
            prefix,
            "".join(random.choice(string.ascii_letters) for i in range(12)),
        )

    ### Expire cleanup ###

    def _clean_expired(self):
        """
        Goes through all messages and groups and removes those that are expired.
        Any channel with an expired message is removed from all groups.
        """
        # Channel cleanup
        for channel, queue in list(self.channels.items()):
            remove = False
            # See if it's expired
            while not queue.empty() and queue._queue[0][0] < time.time():
                queue.get_nowait()
                remove = True
            # Any removal prompts group discard
            if remove:
                self._remove_from_groups(channel)
            # Is the channel now empty and needs deleting?
            if not queue:
                del self.channels[channel]

        # Group Expiration
        timeout = int(time.time()) - self.group_expiry
        for group in self.groups:
            for channel in list(self.groups.get(group, set())):
                # If join time is older than group_expiry end the group membership
                if (
                        self.groups[group][channel]
                        and int(self.groups[group][channel]) < timeout
                ):
                    # Delete from group
                    del self.groups[group][channel]

    ### Flush extension ###

    async def flush(self):
        self.channels = {}
        self.groups = {}

    async def close(self):
        # Nothing to go
        pass

    def _remove_from_groups(self, channel):
        """
        Removes a channel from all groups. Used when a message on it expires.
        """
        for channels in self.groups.values():
            if channel in channels:
                del channels[channel]

    ### Groups extension ###

    async def group_add(self, group, channel):
        """
        Adds the channel name to a group.
        """
        # Check the inputs
        assert self.valid_group_name(group), "Group name not valid"
        assert self.valid_channel_name(channel), "Channel name not valid"
        # Add to group dict
        self.groups.setdefault(group, {})
        self.groups[group][channel] = time.time()

    async def group_discard(self, group, channel):
        # Both should be text and valid
        assert self.valid_channel_name(channel), "Invalid channel name"
        assert self.valid_group_name(group), "Invalid group name"
        # Remove from group set
        if group in self.groups:
            if channel in self.groups[group]:
                del self.groups[group][channel]
            if not self.groups[group]:
                del self.groups[group]

    async def group_send(self, group, message):
        # Check types
        assert isinstance(message, dict), "Message is not a dict"
        assert self.valid_group_name(group), "Invalid group name"
        # Run clean
        self._clean_expired()
        # Send to each channel
        for channel in self.groups.get(group, set()):
            try:
                await self.send(channel, message)
            except ChannelFull:
                pass

# class InMemoryChannelLayer(BaseChannelLayer):
#     """
#     In-memory channel layer implementation
#     """
#
#     def __init__(
#             self,
#             expiry=60,
#             group_expiry=86400,
#             capacity=10000,
#             channel_capacity=None,
#             **kwargs
#     ):
#         super().__init__(
#             expiry=expiry,
#             capacity=capacity,
#             channel_capacity=channel_capacity,
#             **kwargs
#         )
#         self.channels = {}
#         self.groups = {}
#         self.group_expiry = group_expiry
#
#     ### Channel layer API ###
#
#     extensions = ["groups", "flush"]
#
#     def send(self, channel, message):
#         """
#         Send a message onto a (general or specific) channel.
#         """
#         # Typecheck
#         assert isinstance(message, dict), "message is not a dict"
#         assert self.valid_channel_name(channel), "Channel name not valid"
#         # If it's a process-local channel, strip off local part and stick full name in message
#         assert "__asgi_channel__" not in message
#
#         queue = self.channels.setdefault(channel, asyncio.Queue())
#         # Are we full
#         if queue.qsize() >= self.capacity:
#             raise ChannelFull(channel)
#
#         # Add message
#         queue.put((time.time() + self.expiry, deepcopy(message)))
#
#     def receive(self, channel):
#         """
#         Receive the first message that arrives on the channel.
#         If more than one coroutine waits on the same channel, a random one
#         of the waiting coroutines will get the result.
#         """
#         assert self.valid_channel_name(channel)
#         self._clean_expired()
#
#         queue = self.channels.setdefault(channel, asyncio.Queue())
#
#         # Do a plain direct receive
#         _, message = queue.get()
#
#         # Delete if empty
#         if queue.empty():
#             del self.channels[channel]
#
#         return message
#
#     def new_channel(self, prefix="specific."):
#         """
#         Returns a new channel name that can be used by something in our
#         process as a specific channel.
#         """
#         return "%s.inmemory!%s" % (
#             prefix,
#             "".join(random.choice(string.ascii_letters) for i in range(12)),
#         )
#
#     ### Expire cleanup ###
#
#     def _clean_expired(self):
#         """
#         Goes through all messages and groups and removes those that are expired.
#         Any channel with an expired message is removed from all groups.
#         """
#         # Channel cleanup
#         for channel, queue in list(self.channels.items()):
#             remove = False
#             # See if it's expired
#             while not queue.empty() and queue._queue[0][0] < time.time():
#                 queue.get_nowait()
#                 remove = True
#             # Any removal prompts group discard
#             if remove:
#                 self._remove_from_groups(channel)
#             # Is the channel now empty and needs deleting?
#             if not queue:
#                 del self.channels[channel]
#
#         # Group Expiration
#         timeout = int(time.time()) - self.group_expiry
#         for group in self.groups:
#             for channel in list(self.groups.get(group, set())):
#                 # If join time is older than group_expiry end the group membership
#                 if (
#                         self.groups[group][channel]
#                         and int(self.groups[group][channel]) < timeout
#                 ):
#                     # Delete from group
#                     del self.groups[group][channel]
#
#     ### Flush extension ###
#
#     def flush(self):
#         self.channels = {}
#         self.groups = {}
#
#     def close(self):
#         # Nothing to go
#         pass
#
#     def _remove_from_groups(self, channel):
#         """
#         Removes a channel from all groups. Used when a message on it expires.
#         """
#         for channels in self.groups.values():
#             if channel in channels:
#                 del channels[channel]
#
#     ### Groups extension ###
#
#     def group_add(self, group, channel):
#         """
#         Adds the channel name to a group.
#         """
#         # Check the inputs
#         assert self.valid_group_name(group), "Group name not valid"
#         assert self.valid_channel_name(channel), "Channel name not valid"
#         # Add to group dict
#         self.groups.setdefault(group, {})
#         self.groups[group][channel] = time.time()
#
#     def group_discard(self, group, channel):
#         # Both should be text and valid
#         assert self.valid_channel_name(channel), "Invalid channel name"
#         assert self.valid_group_name(group), "Invalid group name"
#         # Remove from group set
#         if group in self.groups:
#             if channel in self.groups[group]:
#                 del self.groups[group][channel]
#             if not self.groups[group]:
#                 del self.groups[group]
#
#     def group_send(self, group, message):
#         # Check types
#         assert isinstance(message, dict), "Message is not a dict"
#         assert self.valid_group_name(group), "Invalid group name"
#         # Run clean
#         self._clean_expired()
#         # Send to each channel
#         for channel in self.groups.get(group, set()):
#             try:
#                 self.send(channel, message)
#             except ChannelFull:
#                 pass



# Generated by Django 3.2.6 on 2021-08-23 13:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0114_auto_20210823_1253'),
    ]

    operations = [
        migrations.AlterField(
            model_name='images',
            name='size',
            field=models.BigIntegerField(default=None, verbose_name='Размер файла'),
        ),
    ]



import logging
from typing import List

from django.db import transaction
from django.db.models import CheckConstraint , F , Q
from isc_common import setAttr , delAttr
from isc_common.common.functions import get_dict_only_model_field
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsBaseRefQuerySet , CommonManagetWithLookUpFieldsManager
from isc_common.models.base_ref import BaseRefHierarcy
from isc_common.models.tree_audit import TreeAuditModelManager
from react.models.fragments_item_types import Fragments_item_types

logger = logging.getLogger( __name__ )


class FragmentsQuerySet( CommonManagetWithLookUpFieldsBaseRefQuerySet ) :
    pass


class FragmentsManager( CommonManagetWithLookUpFieldsManager ) :

    def create4PageFromRequest( self , request , removed=None , propsArr=None ) :
        from react.models.page_fragments import Page_fragments

        request = DSRequest( request=request )
        data = request.get_data()
        with transaction.atomic() :
            if isinstance( data.get( 'fragments' ) , list ) and isinstance( data.get( 'page_id' ) , list ) :
                for page_id in data.get( 'page_id' ) :
                    for fragment_id in data.get( 'fragments' ) :
                        for fragment in Fragments.tree_objects.get_parents( id=fragment_id , child_id='id' , include_self=False ) :
                            Page_fragments.objects.get_or_create( page_id=page_id , fragment=fragment )

        return data

    def updateFromRequest( self , request , removed=None , function=None , propsArr: List = None ) :
        from react.models.fragment_params import Fragment_params

        request = DSRequest( request=request )
        data = request.get_data()

        data = self.check_data_for_multi_select( data=data )
        if data.get( 'mode' ) == 'move' :
            with transaction.atomic() :
                fragment = Fragments.objects.get( id=data.get( 'targetRecord' ) )

                for fragment_param in Fragment_params.objects.filter( id__in=data.get( 'dropRecords' ) ) :
                    fragment_param.fragment = fragment
                    fragment_param.save()
                return data

        _data = data.copy()

        self._remove_prop( data , removed )
        data = self._remove_prop_( data )
        if function :
            function( data )
        else :
            props = self.get_prp( data=data , propsArr=propsArr )
            cloned_data = self.clone_data( data )

            if props is not None :
                setAttr( cloned_data , 'props' , props )

            id = cloned_data.get( 'id' )
            if id is not None :
                delAttr( cloned_data , 'id' )
                parent = cloned_data.get( 'parent' )
                if isinstance( parent , int ) :
                    delAttr( cloned_data , 'parent' )
                    setAttr( cloned_data , 'parent_id' , parent )
                res = super().filter( id=id ).update( **get_dict_only_model_field( data=cloned_data , model=self.model , exclude=[ 'id' ] ) )
                # print(f'res: {res}')
            else :
                new_item = super().create( **cloned_data )
                setAttr( _data , 'id' , new_item.id )
        return _data

    @classmethod
    def getRecord( cls , record ) :
        res = {
            'code' : record.code ,
            'deliting' : record.deliting ,
            'description' : record.description ,
            'editing' : record.editing ,
            'id' : record.id ,
            'name' : record.name ,
            'parent_id' : record.parent.id if record.parent else None ,
            'type_id' : record.type.id ,
            'type__name' : record.type.name ,
        }
        return res

    def get_queryset( self ) :
        return FragmentsQuerySet( self.model , using=self._db )


class Fragments( BaseRefHierarcy ) :
    type = ForeignKeyProtect( Fragments_item_types )

    objects = FragmentsManager()
    tree_objects = TreeAuditModelManager()

    def __str__( self ) :
        return f'ID:{self.id}, code: {self.code}, name: {self.name}, description: {self.description}, type: [{self.type}]'

    def __repr__( self ) :
        return self.__str__()

    class Meta :
        verbose_name = 'Структура фрагмента'
        constraints = [
            CheckConstraint( check=~Q( id=F( 'parent_id' ) ) , name=f'c_Fragments' ) ,
        ]



# Generated by Django 3.1.7 on 2021-03-26 09:48

from django.db import migrations

import isc_common.fields.code_field


class Migration(migrations.Migration):
    dependencies = [
        ('twits', '0019_auto_20190919_1620'),
    ]

    operations = [
        migrations.AlterField(
            model_name='chats',
            name='code',
            field=isc_common.fields.code_field.CodeStrictField(),
        ),
    ]



# Generated by Django 3.2.6 on 2021-08-24 21:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0120_alter_image_types_unique_together'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Users_old_images',
        ),
    ]



from django.urls import path

from one_c.views import documents_param_1c

urlpatterns = [

    path('Documents_param_1c/Fetch/', documents_param_1c.Documents_param_1c_Fetch),
    path('Documents_param_1c/Add', documents_param_1c.Documents_param_1c_Add),
    path('Documents_param_1c/Update', documents_param_1c.Documents_param_1c_Update),
    path('Documents_param_1c/Remove', documents_param_1c.Documents_param_1c_Remove),
    path('Documents_param_1c/Lookup/', documents_param_1c.Documents_param_1c_Lookup),
    path('Documents_param_1c/Info/', documents_param_1c.Documents_param_1c_Info),

]



from isc_common.common.functions import get_relation_field_name
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from tracker.models.messages_theme_users_access import MessagesThemeUsersAccess, MessagesThemeUsersAccessManager


@JsonResponseWithException()
def MessagesThemeUsersAccess_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=MessagesThemeUsersAccess.objects.
                select_related(*get_relation_field_name( model=MessagesThemeUsersAccess )).
                get_range_rows1(
                request=request,
                function=MessagesThemeUsersAccessManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def MessagesThemeUsersAccess_Add(request):
    return JsonResponse(DSResponseAdd(data=MessagesThemeUsersAccess.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def MessagesThemeUsersAccess_Update(request):
    return JsonResponse(DSResponseUpdate(data=MessagesThemeUsersAccess.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def MessagesThemeUsersAccess_Remove(request):
    return JsonResponse(DSResponse(request=request, data=MessagesThemeUsersAccess.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def MessagesThemeUsersAccess_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=MessagesThemeUsersAccess.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def MessagesThemeUsersAccess_Info(request):
    return JsonResponse(DSResponse(request=request, data=MessagesThemeUsersAccess.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def MessagesThemeUsersAccess_Copy(request):
    return JsonResponse(DSResponse(request=request, data=MessagesThemeUsersAccess.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 3.1.6 on 2021-02-20 05:04

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0068_auto_20210219_0916'),
    ]

    operations = [
        migrations.CreateModel(
            name='AuditModelWithOldId',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('old_id', models.BigIntegerField()),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 3.1.7 on 2021-03-10 00:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0078_auto_20210309_0919'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='user',
            name='attfile',
        ),
        migrations.RemoveField(
            model_name='user',
            name='deleted_at',
        ),
        migrations.RemoveField(
            model_name='user',
            name='deliting',
        ),
        migrations.RemoveField(
            model_name='user',
            name='editing',
        ),
        migrations.RemoveField(
            model_name='user',
            name='file_store',
        ),
        migrations.RemoveField(
            model_name='user',
            name='format',
        ),
        migrations.RemoveField(
            model_name='user',
            name='key',
        ),
        migrations.RemoveField(
            model_name='user',
            name='lastmodified',
        ),
        migrations.RemoveField(
            model_name='user',
            name='mime_type',
        ),
        migrations.RemoveField(
            model_name='user',
            name='real_name',
        ),
        migrations.RemoveField(
            model_name='user',
            name='size',
        ),
        migrations.AlterField(
            model_name='user',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



# Generated by Django 3.1.7 on 2021-04-06 10:32

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0091_delete_auditmodelex'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='users_images',
            name='Users_images_unique_constraint_2',
        ),
        migrations.RemoveConstraint(
            model_name='users_images',
            name='Users_images_unique_constraint_3',
        ),
    ]



import logging
import os
import re

from django.conf import settings

logger = logging.getLogger(__name__)


def uncapitalize(str):
    return str[0:1].lower() + str[1:]


def taged(str):
    return '${' + str + '}'


def braces(str):
    return '{' + str + '}'


def get_pattern(str):
    return re.compile('(.+?)(\$?\{' + str + '.*?\})|(.*)')


def capitalize(str):
    return str[0:1].upper() + str[1:]


def dbl_qutes_str(str):
    return f'"{str}"'


def qutes_str(str):
    return f"'{str}'"


def fill_ch(val: str, qty: int, ch='0') -> str:
    if qty == 0:
        return val
    while True:
        val = ch + val
        qty -= 1
        if qty == 0:
            return val


def int_2_bin_str(value: int, _len: int = None) -> str:
    res = ''
    while value > 0:
        res = str(value % 2) + res
        value = value // 2
    if not _len:
        return res
    else:
        return fill_ch(res, _len - len(res))


def reverse_str(value: str) -> str:
    stringlength = len(value)
    return value[stringlength::-1]


class Writer:
    code_type = None
    path_output = None
    template_path = None

    def print_dict(self, **kwargs):
        if isinstance(kwargs, dict):
            logger.debug(f'Class: {self.__class__.__name__}')
            if len(kwargs.items()) == 0:
                logger.debug('{}')
                return
            logger.debug('{')
            for key, value in kwargs.items():
                if isinstance(value, str):
                    logger.debug(f'{self.fill_space(4) + key + " : " + value}')
                elif isinstance(value, bool):
                    logger.debug(f'{self.fill_space(4) + key + " : " + str(value)}')
                elif isinstance(value, int):
                    logger.debug(f'{self.fill_space(4) + key + " : " + str(value)}')
            logger.debug('}')
        else:
            logger.warning(f'{kwargs} is not a dict')

    def check_path(self, path):
        if not os.path.exists(path):
            raise Exception(f'Путь {path} не существует.')
        else:
            return path

    def fill_space(self, qty):
        if qty == -1:
            return ''
        res = ''
        while True:
            res += ' '
            qty -= 1
            if qty == 0:
                return res

    def __init__(self, *args, **kwargs):
        self.path_output = settings.BASE_DIR
        self.check_path(self.path_output)

        self.template_path = settings.BASE_PATH_TEMPLATE
        self.check_path(self.template_path)

    def write_entity(self, replace_map=None):
        o = open(self.path_output, "w+")  # open for append
        for line in open(self.template_path):
            if isinstance(replace_map, dict):
                for key, value in replace_map.items():
                    if isinstance(value, list):
                        value = '\n'.join(value)
                    line = line.replace('${' + key + '}', value)
            o.write(f'{line}')
        o.close()


class Writer:
    code_type = None
    path_output = None
    output_file_ext = None
    output_file_name = None
    template_path = None

    def print_dict(self, **kwargs):
        if isinstance(kwargs, dict):
            logger.debug(f'Class: {self.__class__.__name__}')
            if len(kwargs.items()) == 0:
                logger.debug('{}')
                return
            logger.debug('{')
            for key, value in kwargs.items():
                if isinstance(value, str):
                    logger.debug(f'{self.fill_space(4) + key + " : " + value}')
                elif isinstance(value, bool):
                    logger.debug(f'{self.fill_space(4) + key + " : " + str(value)}')
                elif isinstance(value, int):
                    logger.debug(f'{self.fill_space(4) + key + " : " + str(value)}')
            logger.debug('}')
        else:
            logger.warning(f'{kwargs} is not a dict')

    def check_path(self, path):
        if not os.path.exists(path):
            raise Exception(f'Путь {path} не существует.')
        else:
            return path

    def fill_space(self, qty):
        if qty == -1:
            return ''
        res = ''
        while True:
            res += ' '
            qty -= 1
            if qty == 0:
                return res

    def __init__(self, *args, **kwargs):
        self.path_output = kwargs.get('path_output')
        self.output_file_name = kwargs.get('output_file_name')
        self.output_file_ext = kwargs.get('output_file_ext')
        self.template_path = kwargs.get('template_path')

    def write_entity(self, replace_map=None):
        path_output = self.path_output
        if self.output_file_name:
            path_output += os.sep + self.output_file_name
        if self.output_file_ext:
            path_output += self.output_file_ext

        o = open(path_output, "w+")  # open for append
        for line in open(self.template_path):
            if isinstance(replace_map, dict):
                for key, value in replace_map.items():
                    if isinstance(value, list):
                        value = '\n'.join(value)
                    line = line.replace('${' + key + '}', value)
            o.write(f'{line}')
        o.close()



# Generated by Django 3.2.7 on 2021-09-13 10:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0131_text_informations_props'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='images',
            unique_together={('file_name', 'size'), ('real_name', 'size')},
        ),
    ]



# Generated by Django 3.1.6 on 2021-02-17 14:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0062_auto_20210217_1442'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='password',
            field=models.CharField(max_length=255, verbose_name='пароль'),
        ),
    ]



from django.urls import path

from react.views import pages_view

urlpatterns = [

    path('Pages_view/Fetch/', pages_view.Pages_view_Fetch),
    path('Pages_view/Add', pages_view.Pages_view_Add),
    path('Pages_view/Update', pages_view.Pages_view_Update),
    path('Pages_view/Remove', pages_view.Pages_view_Remove),
    path('Pages_view/Lookup/', pages_view.Pages_view_Lookup),
    path('Pages_view/Info/', pages_view.Pages_view_Info),
    path('Pages_view/Copy', pages_view.Pages_view_Copy),

]



# Generated by Django 2.2.6 on 2019-10-12 09:15

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0069_messages_importance'),
    ]

    operations = [
        migrations.AddField(
            model_name='messages',
            name='importance',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='tracker.Messages_importance'),
        ),
    ]



from django.urls import path

from refs.views import type_param

urlpatterns = [

    path('Type_param/Fetch/', type_param.Type_param_Fetch),
    path('Type_param/Add', type_param.Type_param_Add),
    path('Type_param/Update', type_param.Type_param_Update),
    path('Type_param/Remove', type_param.Type_param_Remove),
    path('Type_param/Lookup/', type_param.Type_param_Lookup),
    path('Type_param/Info/', type_param.Type_param_Info),
    path('Type_param/Copy', type_param.Type_param_Copy),

]



from django.urls import path

from react.views import pages

urlpatterns = [

    path('Pages/Fetch/', pages.Pages_Fetch),
    path('Pages/Add', pages.Pages_Add),
    path('Pages/Update', pages.Pages_Update),
    path('Pages/Remove', pages.Pages_Remove),
    path('Pages/Lookup/', pages.Pages_Lookup),
    path('Pages/Info/', pages.Pages_Info),
    path('Pages/Generate', pages.Pages_Generate),
    path('Pages/Copy', pages.Pages_Copy),

]



import logging

from django.conf import settings
from django.db.models import TextField, DecimalField, BooleanField, SmallIntegerField

from isc_common.fields.name_field import NameField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.base_ref import BaseRef, BaseRefManager, BaseRefQuerySet
from isc_common.models.images import Images
from react.models.fragment_param_types import Fragment_param_types
from react.models.fragment_params import Fragment_paramsManager
from react.models.fragments import Fragments

logger = logging.getLogger(__name__)


class Fragment_params_viewQuerySet(BaseRefQuerySet):
    pass


class Fragment_params_viewManager(BaseRefManager):

    @classmethod
    def getRecord(cls, record ) :
        image_src = f'{settings.IMAGE_CONTENT_PROTOCOL}://{settings.IMAGE_CONTENT_HOST}:{settings.IMAGE_CONTENT_PORT}/{record.image_src}&ws_host={settings.WS_HOST}&ws_port={settings.WS_PORT}&ws_channel={settings.WS_CHANNEL}'

        value_text = record.value_text
        if not value_text and record.image is not None:
            value_text = image_src
            Fragment_params_view.objects.filter(id=record.id).update(value_text=value_text)

        res = {
            'code': record.code,
            'deliting': record.deliting,
            'description': record.description,
            'editing': record.editing,
            'fragment_id': record.fragment.id,
            'id': record.id,
            'image_id': record.image.id if record.image else None,
            'image_real_name': record.image_real_name,
            'image_src': image_src,
            'name': record.name,
            'num': record.num,
            'path_file': record.path_file,
            'props': record.props,
            'type__name': record.type.name,
            'type_id': record.type.id,
            'use_generator': record.use_generator,
            'value_dec': record.value_dec,
            'value_text': value_text,
        }
        return res

    def get_queryset(self):
        return Fragment_params_viewQuerySet(self.model, using=self._db)


class Fragment_params_view(BaseRef):
    fragment = ForeignKeyProtect(Fragments)
    image = ForeignKeyProtect(Images, null=True, blank=True)
    image_real_name = NameField(null=True, blank=True)
    image_src = NameField()
    num = SmallIntegerField(null=True, blank=True)
    path_file = TextField(null=True, blank=True)
    props = Fragment_paramsManager.props()
    type = ForeignKeyProtect(Fragment_param_types)
    use_generator = BooleanField()
    value_dec = DecimalField(decimal_places=4, max_digits=19, null=True, blank=True)
    value_text = TextField(null=True, blank=True)

    objects = Fragment_params_viewManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Параметры фрагмента'
        db_table = 'react_fragment_params_view'
        managed = False



# Generated by Django 3.2.3 on 2021-05-14 15:27

from django.db import migrations
import isc_common.fields.files


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0089_alter_messages_files_real_name'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messages_files',
            name='attfile',
            field=isc_common.fields.files.FileFieldEx(default=None, max_length=255, upload_to='', verbose_name='Файл'),
            preserve_default=False,
        ),
    ]



import logging

from django.contrib.postgres.fields import ArrayField
from django.db.models import BigIntegerField

from isc_common.auth.models.user import User
from isc_common.fields.code_field import CodeField
from isc_common.fields.name_field import NameField
from isc_common.models.audit import AuditManager , AuditQuerySet

logger = logging.getLogger( __name__ )


class UserViewQuerySet( AuditQuerySet ) :
    pass


class UserViewManager( AuditManager ) :

    @classmethod
    def getRecord( cls , **kwargs ) :
        record = kwargs.get( 'record' )
        res = {
            'deliting' : record.deliting ,
            'description' : record.description ,
            'e_mails' : '<br>'.join( record.e_mails ) ,
            'editing' : record.editing ,
            'first_name' : record.first_name ,
            'id' : record.id ,
            'last_login' : record.last_login ,
            'last_name' : record.last_name ,
            'lastmodified' : record.lastmodified ,
            'middle_name' : record.middle_name ,
            'password' : record.password ,
            'phones' : '<br>'.join( record.phones ) ,
            'username' : record.username ,
        }
        return res

    def get_queryset( self ) :
        return UserViewQuerySet( self.model , using=self._db )


class UserView( User ) :
    e_mail_ids = ArrayField( BigIntegerField() , null=True , blank=True , )
    e_mails = ArrayField( CodeField() , null=True , blank=True , )
    full_user_name = NameField()
    phone_ids = ArrayField( BigIntegerField() , null=True , blank=True , )
    phones = ArrayField( CodeField() , null=True , blank=True , )
    short_user_name = NameField()

    objects = UserViewManager()

    class Meta :
        verbose_name = 'Пользователь'
        managed = False
        db_table = 'isc_common_user_view'



# Generated by Django 3.2.6 on 2021-08-31 22:39
import django
from django.db import migrations

import isc_common


class Migration( migrations.Migration ) :
    dependencies = [
        ('isc_common' , '0125_image_types_c_event') ,
        ('competitions', '0360_auto_20210831_2135'),
    ]

    operations = [
        migrations.AlterField(
            model_name='users_images' ,
            name='type' ,
            field=isc_common.fields.related.ForeignKeyProtect( default=None , on_delete=django.db.models.deletion.PROTECT , to='isc_common.image_types' ) ,
            preserve_default=False ,
        ) ,
    ]



import logging
import os
import shutil
import xml.etree.ElementTree as ET

from django.conf import settings

from isc_common import setAttr, Stack, StackElementNotExist
from isc_common.ws.webSocket import WebSocket
from react.models.page_fragments import Page_fragments

logger = logging.getLogger(__name__)


class AbstractGenerator:
    gen_variables = dict()
    gen_files = Stack()


class TypeScriptGenerator(AbstractGenerator):
    def _get_text(self, text):
        if isinstance(text, str):
            try:
                root = ET.fromstring(text=text)
            except ET.ParseError:
                return text
        else:
            root = text

        if len(list(root)) == 0:
            return root.text

        for child in root:
            res = self._get_text(child)
            if isinstance(res, str):
                return res

        raise Exception('Not found text/')

    def _rec_gen_files(self):
        for path_file, data_4_file in self.gen_files:
            pathes_file = path_file.split('/')
            _pathes_file = []
            for p in pathes_file:
                if '$' in p:
                    p_item = self.gen_variables.get(p.replace('$', ''))
                    if p_item is None:
                        raise Exception(f'Variable {p} not found.')

                    _pathes_file.append(p_item.get('value_text'))
                else:
                    _pathes_file.append(p)
            pathes_file = '/'.join(_pathes_file)
            dir, file = os.path.split(pathes_file)

            if os.path.exists(dir) and 'generated' in dir:
                shutil.rmtree(dir)
                os.mkdir(dir)

            if not os.path.exists(dir):
                os.mkdir(dir)

            with open(pathes_file, 'w') as f:
                for line in data_4_file:
                    f.write(line.replace('&amp;', '&'))
                    f.write('\n')

        WebSocket.send_info_message(
            host=settings.WS_HOST,
            port=settings.WS_PORT,
            channel=f'common_{self.user.username}',
            message='Генерация выполнена',
            logger=logger
        )

    def _get_comment(self, text):
        return f'/* {self._get_text(text)} */'

    def _set_file_text(self, data_4_file: list, fragment_param):
        if not isinstance(data_4_file, list):
            raise Exception(f'file must be list.')

        if 'SCSS' in fragment_param.type.code and 'VAR' in fragment_param.type.code and 'URL' in fragment_param.type.code:
            data_4_file.append(self._get_comment(fragment_param.description))
            data_4_file.append(f'${fragment_param.code} : "{self._get_text(fragment_param.value_text)}";')

        if fragment_param.type.code == 'TS_MODULE':
            data_4_file.append(self._get_comment(fragment_param.description))
            data_4_file.append(fragment_param.value_text)

    def generate(self, data, user):
        from react.models.pages import Pages
        from react.models.fragment_params import Fragment_params

        self.user = user
        for pages_id in data.get('pages_ids'):
            for page in Pages.tree_objects.get_descendants(id=pages_id, child_id='id', order_by_clause='order by level'):
                for page_fragment in Page_fragments.objects.filter(page=page):
                    for fragment_param in Fragment_params.objects.filter(fragment=page_fragment.fragment).order_by('num'):
                        if fragment_param.type.code == 'GEN_VAR':
                            if self.gen_variables.get(fragment_param.code) is None:
                                setAttr(self.gen_variables, fragment_param.code, dict(value_dec=fragment_param.value_dec, value_text=fragment_param.value_text))
                        elif fragment_param.props.use_generator.is_set is True:
                            try:
                                _, data_4_file = self.gen_files.find_one(lambda x: x[0] == fragment_param.path_file)
                                self._set_file_text(data_4_file=data_4_file, fragment_param=fragment_param)
                            except StackElementNotExist:
                                data_4_file = []
                                self._set_file_text(data_4_file=data_4_file, fragment_param=fragment_param)
                                self.gen_files.push((fragment_param.path_file, data_4_file))
        self._rec_gen_files()



import logging

from django.forms import model_to_dict

from isc_common import setAttr, delAttr, dictinct_list
from isc_common.auth.managers.base_user_manager import BaseUserManager, BaseUserQuerySet
from isc_common.auth.models.usergroup import UserGroup
from isc_common.bit import IsBitOn
from isc_common.common.functions import get_dict_only_model_field
from isc_common.http.DSRequest import DSRequest

logger = logging.getLogger(__name__)


class UserQuerySet(BaseUserQuerySet):
    pass


class UserManager(BaseUserManager):
    use_in_migrations = True

    def get_or_create(self, defaults=None, **kwargs):
        from isc_common.auth.models.user import User

        if defaults is None:
            defaults = kwargs

        username = kwargs.get('username')
        if username is None:
            username = defaults.get('username')
        if username is None:
            username = User.full_name(
                first_name=defaults.get('first_name'),
                last_name=defaults.get('last_name'),
                middle_name=defaults.get('middle_name'),
                birthday=defaults.get('birthday'),
                translit=True,
            )

            if username is None:
                raise Exception('username is None')

        user = User.objects.getOptional(username=username)
        if user is None:
            description = defaults.get('description')

            if description is not None and description.strip() == '':
                description = None

            setAttr(defaults, 'description', description)
            user_data = self.get_user_data(**defaults)
            try:
                user = super().get(username=username)
                return user, False

            except User.DoesNotExist as ex:
                setAttr(user_data, 'username', username)
                user = super().create(**user_data)
                user.set_password(username)
                user.save()

                return user, True


        else:
            return user, False

    def update_or_create(self, defaults=None, **kwargs):
        return super().update_or_create(defaults=defaults, **kwargs)

    @classmethod
    def get_user_data(cls, **kwargs):
        last_name = kwargs.get('last_name')
        first_name = kwargs.get('first_name')
        middle_name = kwargs.get('middle_name')
        description = kwargs.get('description')
        birthday = kwargs.get('birthday')
        # username = kwargs.get('username')

        user_data = dict(
            last_name=last_name.replace('ё', 'e').strip() if last_name else None,
            first_name=first_name.replace('ё', 'e').strip() if first_name else None,
            middle_name=middle_name.replace('ё', 'e').strip() if middle_name else None,
            birthday=birthday if birthday else None,
            description=description,
            # username=username
        )

        return user_data

    def get_queryset(self):
        return UserQuerySet(self.model, using=self._db).filter(deleted_at=None)

    @classmethod
    def getRecord(cls, **kwargs):
        record = kwargs.get('record')
        res = {
            'birthday': record.birthday,
            'bot': IsBitOn(record.props, 0),
            'color': record.color,
            'deliting': record.deliting,
            'description': record.description,
            'editing': record.editing,
            'email': record.email,
            'first_name': record.first_name,
            'id': record.id,
            'last_login': record.last_login,
            'last_name': record.last_name,
            'lastmodified': record.lastmodified,
            'middle_name': record.middle_name,
            'password': record.password,
            'short_name': record.get_short_name,
            'short_name1': record.get_short_name1,
            'username': record.username,
        }
        return res

    @classmethod
    def getRecord1(cls, record):
        return cls.getRecord(**record)

    def updateFromRequest(self, request):
        from isc_common.auth.models.user import User

        request = DSRequest(request=request)
        data = request.get_data()

        data = self.check_data_for_multi_select(data=data)
        _data = data.copy()

        delAttr(data, 'usergroup')
        oldValues = request.get_oldValues()

        _oldValues = oldValues.get('data')
        if not _oldValues:
            _oldValues = oldValues

        _oldValues = self.check_data_for_multi_select(data=_oldValues)

        data = self._remove_prop_(data)
        values = [item for item in dictinct_list(set(_oldValues) - set(data)) if not item.startswith('_')]
        for item in values:
            setAttr(data, item, None)

        user_id = request.get_id()
        if user_id is None:
            user_id = data.get('id')

        super().filter(id=user_id).update(**get_dict_only_model_field(data=data, model=User, exclude=['password']))

        user = self.model.objects.get(id=user_id)
        if user.check_password(data.get('password')) is False:
            user.set_password(data.get('password', None))
            user.save()
            setAttr(data, 'password', user.password)
            setAttr(data, 'short_name', user.get_short_name)

        return _data

    def createFromRequest(self, request):
        from isc_common.auth.models.user import User

        request = DSRequest(request=request)
        data = request.get_data(excluded_keys=['id'])

        usergroup_id = data.get('usergroup_id')
        delAttr(data, 'usergroup_id')
        delAttr(data, 'on_line')

        kwarg = self.clone_data(data=data, model=User)
        user = User.objects.create(**kwarg)
        password = request.get_data(excluded_keys=['id']).get('password')

        user.set_password(password)
        user.save(using=self._db)

        user = User.objects.get(pk=user.id)
        if isinstance(usergroup_id, list):
            for group in usergroup_id:
                user.usergroup.add(group)

        res = model_to_dict(user)
        setAttr(res, 'props', res.get('props')._value)

        return res

    def _create_user(self, usergroup, username, email, password, **extra_fields):
        if not username:
            raise ValueError('The given username must be set')

        email = self.normalize_email(email)
        username = self.model.normalize_username(username)
        user = self.model(username=username, email=email, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)

        if isinstance(usergroup, list):
            user.usergroup.set(usergroup)
        elif isinstance(usergroup, UserGroup):
            user.usergroup.set([usergroup])

        return user

    def create_user(self, usergroup, username, email=None, password=None, **extra_fields):
        try:
            return self.model.objects.get(username=username)
        except self.model.DoesNotExist:
            return self._create_user(usergroup, username, email, password, **extra_fields)

    def create_superuser(self, username, email, password, **extra_fields):
        try:
            return self.model.objects.get(username=username)
        except self.model.DoesNotExist:
            return self._create_user(usergroup=UserGroup.objects.get(code='administrators'), username=username, email=email, password=password, **extra_fields)



# Generated by Django 2.2 on 2019-04-21 15:01

from django.conf import settings
from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0016_messages_view'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messages',
            name='user',
            field=isc_common.fields.related.ForeignKeyCascade(default=None, on_delete=django.db.models.deletion.CASCADE, related_name='user', to=settings.AUTH_USER_MODEL),
        ),
    ]



import logging

from django.core.management import BaseCommand

logger = logging.getLogger(__name__)

import logging
import tqdm


class TqdmLoggingHandler(logging.Handler):
    def __init__(self, level=logging.NOTSET):
        super().__init__(level)

    def emit(self, record):
        try:
            msg = self.format(record)
            tqdm.tqdm.write(msg)
            self.flush()
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)


class Command(BaseCommand):
    def handle(self, *args, **options):
        logger.addHandler(TqdmLoggingHandler())
        for i in tqdm.tqdm(range(100)):
            logger.info(i)



# Generated by Django 3.1.7 on 2021-03-23 05:53

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0087_auto_20210316_0827'),
    ]

    operations = [
        migrations.RenameField(
            model_name='user',
            old_name='birtdate',
            new_name='birthday',
        ),
    ]



import logging

from bitfield import BitField

from isc_common.fields.code_field import CodeStrictField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditManager
from isc_common.models.e_mails import E_mails, Model_e_mailQuerySet
from lfl_admin.user_ext.models.persons import User

logger = logging.getLogger(__name__)


class User_e_mailsQuerySet(Model_e_mailQuerySet):
    pass


class User_e_mailsManager(AuditManager):
    @classmethod
    def props(cls):
        return BitField(flags=(
            ('main', 'Главная'),  # 1
        ), default=0, db_index=True)

    @classmethod
    def getRecord(cls, record ) :
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return User_e_mailsQuerySet(self.model, using=self._db)


class User_e_mails(AuditModel):
    code = CodeStrictField()
    user = ForeignKeyProtect(User)
    e_mail = ForeignKeyProtect(E_mails)
    props = User_e_mailsManager.props()

    objects = User_e_mailsManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Администрация клуба'
        unique_together = (('code', 'user', 'e_mail'),)



from django.urls import path

from react.views import fragment_params

urlpatterns = [

    path('Fragment_params/Fetch/', fragment_params.Fragment_params_Fetch),
    path('Fragment_params/Add', fragment_params.Fragment_params_Add),
    path('Fragment_params/Update', fragment_params.Fragment_params_Update),
    path('Fragment_params/Remove', fragment_params.Fragment_params_Remove),
    path('Fragment_params/Lookup/', fragment_params.Fragment_params_Lookup),
    path('Fragment_params/Info/', fragment_params.Fragment_params_Info),
    path('Fragment_params/Copy', fragment_params.Fragment_params_Copy),
    path('Fragment_params/UploadImage', fragment_params.Fragment_params_ImagesUpload),

]



class WsLogger:
    def set_font(self, text, color='black', size=2, face='verdana'):
        return f'<font color="{color}" size="{size}" face="{face}">{text}</font>'

    def get_format_message(self, text, mode='debug'):
        if mode == 'debug':
            color = 'green'
        elif mode == 'error':
            color = 'red'
        elif mode == 'info':
            color = 'blue'
        elif mode == 'warning':
            color = 'orange'
        else:
            color = 'black'
        return f'<span><b>{self.set_font(mode.upper(), color)}</b>: {text}<span>'



import logging
import sys
import traceback
from contextlib import contextmanager

from django.conf import settings
from django.db import transaction
from django.utils import timezone
from tqdm import tqdm

from isc_common.bit import IsBitOff
from isc_common.models.deleted_progresses import Deleted_progresses
from isc_common.models.progresses import Progresses
from isc_common.ws.progressStack import ProgressStack

logger = logging.getLogger(__name__)

progress_deleted = 'Процесс удален.'


class ProgressDroped(Exception):
    ...


@contextmanager
def managed_progress(qty, user, id, message='Обработано позиций', title='Обработано позиций', props=0, sendError=True):
    progress = Progress(id=id, qty=qty, user=user, message=message, title=title, props=props)
    try:
        yield progress
        progress.close()
    except ProgressDroped as ex:
        progress.close()
        if callable(progress.except_func):
            progress.except_func()
        Deleted_progresses.objects.filter(id_progress=id, user=user).delete()
        raise ex
    except Exception as ex:
        if callable(progress.except_func):
            progress.except_func()
        exc_info = sys.exc_info()
        stackTrace = traceback.format_exception(*exc_info)
        stackTrace = '<br>'.join(stackTrace)
        stackTrace = f'<pre>{stackTrace}</pre>'

        if sendError is True:
            progress.sendError(message=str(ex), stackTrace=stackTrace)
        else:
            logger.exception(msg=str(ex), exc_info=exc_info)

        del exc_info

        progress.close()
        raise ex


@contextmanager
def managed_progreses(qty, users, id, message='Обработано позиций', title='Обработано позиций', props=0):
    progreses = Progreses(id=id, qty=qty, users=users, message=message, title=title, props=props)
    try:
        yield progreses
        progreses.close()
    except ProgressDroped as ex:
        progreses.close()
        if callable(progreses.except_func):
            progreses.except_func()
        for progress in progreses.progreses:
            Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
        raise ex
    except Exception as ex:
        if callable(progreses.except_func):
            progreses.except_func()
        exc_info = sys.exc_info()
        stackTrace = traceback.format_exception(*exc_info)
        stackTrace = '<br>'.join(stackTrace)
        stackTrace = f'<pre>{stackTrace}</pre>'
        del exc_info
        progreses.sendError(message=str(ex), stackTrace=stackTrace)

        progreses.close()
        raise ex


class Progreses:
    progreses = None

    def __init__(self, users, id, qty=0, message='Обработано позиций', title='Обработано позиций', label_contents=None, props=0):
        self.progreses = list(map(lambda user: Progress(id=f'{id}_{user.id}', qty=qty, user=user, message=message, title=title, label_contents=label_contents, props=props), users))

    def step(self):
        for progress in self.progreses:
            res = progress.step()
            if res != 0:
                return res
        else:
            return 0

    def setContentsLabel(self, content):
        for progress in self.progreses:
            progress.setContentsLabel(content)

    def setQty(self, qty):
        for progress in self.progreses:
            progress.setQty(qty)

    def sendInfo(self, message):
        for progress in self.progreses:
            progress.sendInfo(message)

    def sendWarn(self, message):
        for progress in self.progreses:
            progress.sendWarn(message)

    def sendError(self, message, stackTrace=None):
        for progress in self.progreses:
            progress.sendError(message, stackTrace)

    def sendMessage(self, type, message=None):
        for progress in self.progreses:
            progress.sendMessage(type, message)

    def close(self):
        for progress in self.progreses:
            progress.close()


class Progress:
    progress = None
    pbar = None
    except_func = None
    enable_ws_progress = None

    def __init__(self, user, id, qty=0, message='Обработано позиций', title='Обработано позиций', label_contents=None, props=0):
        from isc_common.auth.models.user import User

        if isinstance(message, tuple):
            raise Exception(f'{message} is tuple')

        self.message = message
        self.title = title
        self.props = props
        self.id = id
        self.enable_ws_progress = IsBitOff(self.props, 1)

        if self.enable_ws_progress is True:
            if isinstance(user, int):
                self.user = User.objects.get(id=user)
            elif isinstance(user, User):
                self.user = user
            else:
                raise Exception(f'user must be User instance or User_id.')

        self.qty = qty

        if self.enable_ws_progress == False:
            self.pbar = tqdm(total=qty)
        else:
            # if self.qty > 0:
            channel = f'common_{self.user.username}'
            self.progress = ProgressStack(
                host=settings.WS_HOST,
                port=settings.WS_PORT,
                channel=channel,
                props=self.props,
                user_id=self.user.id
            )

            self.demand_str = f'<h3>{self.message}</h3>'

            # if not self.check_exists():

            deleted, _ = Deleted_progresses.objects.filter(id_progress=self.id, user=self.user).delete()
            logger.debug(f'Deleted_progresses: {deleted}')

            self.progress.show(
                cntAll=qty,
                id=self.id,
                label_contents=label_contents if label_contents else self.demand_str,
                title=f'<b>{title}</b>',
            )

            # if IsBitOff(props, 1):
            self.progresses, created = Progresses.objects.get_or_create(
                id_progress=self.id,
                user=self.user,
                defaults=dict(
                    cnt=0,
                    label_contents=self.demand_str,
                    qty=self.qty,
                    message=self.message,
                    props=self.props,
                    title=self.title,
                    lastmodified=timezone.now()
                )
            )

        self.cnt = 0

    def check_exists(self):
        return Progresses.objects.filter(id_progress=self.id, user=self.user).count() > 0

    def step(self):
        from isc_common.models.deleted_progresses import Deleted_progresses

        res = 0
        if self.enable_ws_progress is True:
            if self.progress is not None:
                self.cnt += 1
                self.progress.setCntDone(self.cnt, self.id)
                if self.qty == self.cnt:
                    self.cnt = 0
                self.progresses.cnt = self.cnt
                self.progresses.lastmodified = timezone.now()
                self.progresses.save()
                deleted_count = Deleted_progresses.objects.filter(id_progress=self.id, user=self.user).count()
                return deleted_count
        else:
            self.pbar.update()
            logger.debug(str(self.pbar))
        return res

    def setContentsLabel(self, content):
        if self.progress is not None:
            self.progress.setContentsLabel(labelContents=content, id=self.id)
            with transaction.atomic():
                self.progresses.label_contents = content
                self.progresses.save()

    def setQty(self, qty):
        if self.progress is not None:
            self.qty = qty
            self.progress.setCntAll(cntAll=qty, id=self.id)
            with transaction.atomic():
                self.progresses.qty = qty
                self.progresses.save()

    def sendInfo(self, message):
        if self.progress is not None:
            self.progress.sendInfo(message=message)

    def sendWarn(self, message):
        if self.progress is not None:
            self.progress.sendWarn(message=message)

    def sendError(self, message, stackTrace=None):
        if self.progress is not None:
            self.progress.sendError(message=message, stackTrace=stackTrace)

    def sendMessage(self, type, message=None):
        if self.progress is not None:
            self.progress.sendMessage(type=type, message=message)

    def close(self):
        if self.progress is not None:
            self.progress.close(self.id)
            try:
                self.progresses.delete()
            except AssertionError:
                pass

        if self.pbar:
            self.pbar.close()



import logging

from django.core.management import BaseCommand

from isc_common.common.mat_views import create_insert_update_delete_function_of_table

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('--tablename', type=str)

    def handle(self, *args, **options):
        tablename = options.get('tablename')

        create_insert_update_delete_function_of_table(table_name=tablename)



# Generated by Django 3.2.6 on 2021-08-23 07:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0106_auto_20210823_0721'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='users_images',
            unique_together={('image', 'main_model', 'props', 'type', 'deleted_at')},
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-24 19:24

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0050_auto_20190824_1921'),
    ]

    operations = [
        migrations.AlterField(
            model_name='calendar_shifts_days',
            name='friday',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='friday_rel', to='clndr.Shift_day'),
        ),
        migrations.AlterField(
            model_name='calendar_shifts_days',
            name='monday',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='monday_rel', to='clndr.Shift_day'),
        ),
        migrations.AlterField(
            model_name='calendar_shifts_days',
            name='saturday',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='saturday_rel', to='clndr.Shift_day'),
        ),
        migrations.AlterField(
            model_name='calendar_shifts_days',
            name='sunday',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='sunday_rel', to='clndr.Shift_day'),
        ),
        migrations.AlterField(
            model_name='calendar_shifts_days',
            name='thursday',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='thursday_rel', to='clndr.Shift_day'),
        ),
        migrations.AlterField(
            model_name='calendar_shifts_days',
            name='tuesday',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='tuesday_rel', to='clndr.Shift_day'),
        ),
        migrations.AlterField(
            model_name='calendar_shifts_days',
            name='wednesday',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='wednesday_rel', to='clndr.Shift_day'),
        ),
    ]



# Generated by Django 3.1.6 on 2021-02-11 07:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0082_auto_20210211_0715'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messages',
            name='message',
            field=models.TextField(),
        ),
    ]



# Generated by Django 3.0.5 on 2020-04-28 10:25

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Files_group',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.TextField(db_index=True)),
            ],
            options={
                'verbose_name': 'Группы файлов',
            },
        ),
        migrations.CreateModel(
            name='Files',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('path', models.TextField(db_index=True)),
                ('group', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='files_blk.Files_group')),
            ],
            options={
                'verbose_name': 'Файлы',
            },
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-21 16:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0015_day_types'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='holidays',
            name='props',
        ),
    ]



import logging

from django.db.models import Model

from isc_common.auth.models.user import User
from isc_common.fields.related import ForeignKeyCascade
from isc_common.models.audit import AuditQuerySet, AuditManager
from tracker.models.messages import Messages

logger = logging.getLogger(__name__)


class Messages_whomQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Messages_whomManager(AuditManager):

    @classmethod
    def getRecord(cls, record ) :
        res = {}
        return res

    def get_queryset(self):
        return Messages_whomQuerySet(self.model, using=self._db)


class Messages_whom(Model):
    message = ForeignKeyCascade(Messages)
    to_whom = ForeignKeyCascade(User)

    objects = Messages_whomManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Получатель сообщения'
        unique_together = (('message', 'to_whom'),)



# Generated by Django 3.0.2 on 2020-01-22 07:38

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0033_standard_colors'),
    ]

    operations = [
        migrations.AddField(
            model_name='standard_colors',
            name='color',
            field=isc_common.fields.code_field.ColorField(),
        ),
    ]



# Generated by Django 3.2.6 on 2021-08-11 17:36

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('react', '0020_pages_props'),
    ]

    operations = [
        migrations.AddField(
            model_name='fragment_params',
            name='props',
            field=bitfield.models.BitField((('use_generator', 'Используется генератором'),), db_index=True, default=0),
        ),
        migrations.AlterField(
            model_name='pages',
            name='props',
            field=bitfield.models.BitField((('application', 'Приложение'),), db_index=True, default=0),
        ),
    ]



import logging

from django.conf import settings
from django.db import transaction

from events.models.event_types import Event_types
from isc_common import delAttr
from isc_common.auth.models.user import User
from isc_common.bit import IsBitOn, TurnBitOn
from isc_common.fields.related import ForeignKeyCascade, ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from twits.models.chat_user_user import Chat_user_user
from twits.models.chats import ChatsManager

logger = logging.getLogger(__name__)


class Event_type_usersQuerySet(AuditQuerySet):
    pass


class Event_type_usersManager(AuditManager):

    @classmethod
    def get_bot(cls, event_type):
        if not isinstance(event_type, Event_types):
            raise Exception(f'event_type must be a Event_types')

        key = 'Event_type_usersManager.get_bot'
        settings.LOCKS.acquire(key)
        bot, _ = User.objects.get_or_create(
            username=f'event_{event_type.code}_bot',
            defaults=dict(
                last_name=f'Бот ({event_type.full_name})',
                password='1234567890',
                props=User.props.bot,
                deliting=False,
                editing=False
            )
        )
        settings.LOCKS.release(key)
        return bot

    @classmethod
    def getRecord(cls, record):
        res = {
            'id': record.id,
            'event_type_id': record.event_type.id,
            'user_id': record.user.id,
            'user__username': record.user.username,
            'user__first_name': record.user.first_name,
            'user__last_name': record.user.last_name,
            'user__email': record.user.email,
            'user__middle_name': record.user.middle_name,
            'props': record.user.props._value,
        }
        return res

    def get_queryset(self):
        return Event_type_usersQuerySet(self.model, using=self._db)

    def updateFromRequest(self, request, removed=None, function=None):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()

        delAttr(_data, 'id')
        delAttr(_data, 'user__username')
        delAttr(_data, 'user__first_name')
        delAttr(_data, 'user__last_name')
        delAttr(_data, 'user__email')
        delAttr(_data, 'user__middle_name')

        res = super().filter(id=data.get('id')).update(**_data)
        return data

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        user_ids = data.get('user_ids', None)
        context_ids = data.get('context_ids', None)

        with transaction.atomic():
            if user_ids and context_ids:
                if not isinstance(user_ids, list):
                    user_ids = [user_ids]

                if not isinstance(context_ids, list):
                    context_ids = [context_ids]

                for user_id in user_ids:
                    for context_id in context_ids:
                        event_types = Event_types.objects.get(id=context_id)
                        if not IsBitOn(event_types.props, 0):
                            raise Exception(f'Строка: {event_types.full_name} не является событием, и не может подлежать подписке.')
                        res, _ = super().get_or_create(user_id=user_id, event_type_id=context_id)

                        props = 0
                        if IsBitOn(res.event_type.props, 1):
                            props = TurnBitOn(props, 0)
                        chat, created = Chat_user_user.objects.get_or_create(user_reciver_id=user_id, user_sender=Event_type_usersManager.get_bot(res.event_type), defaults=dict(props=props))
                        ChatsManager.refresh_chat_menu(user=User.objects.get(id=user_id), logger=logger)

            return data


class Event_type_users(AuditModel):
    event_type = ForeignKeyCascade(Event_types)
    user = ForeignKeyCascade(User)

    objects = Event_type_usersManager()

    def __str__(self):
        return f"ID:{self.id}, event_type: [{self.event_type}], user: [{self.user}]"

    class Meta:
        verbose_name = 'Подписки на события'
        unique_together = (('event_type', 'user'),)



import logging

from django.utils.crypto import get_random_string

from isc_common.models.audit import AuditQuerySet, AuditManager

logger = logging.getLogger(__name__)


class BaseUserQuerySet(AuditQuerySet):
    def getRecord(self, record):
        res = {
            "id": record.id,
            "username": record.username,
            "first_name": record.first_name,
            "last_name": record.last_name,
            "middle_name": record.middle_name,
            "email": record.email,
            "password": record.password,
            "last_login": record.last_login,
            "deliting": record.deliting,
            "editing": record.editing,
        }
        return res


class BaseUserManager(AuditManager):

    def get_queryset(self):
        return BaseUserQuerySet(self.model, using=self._db).filter(deleted_at=None)

    @classmethod
    def normalize_email(cls, email):
        """
        Normalize the email address by lowercasing the domain part of it.
        """
        email = email or ''
        try:
            email_name, domain_part = email.strip().rsplit('@', 1)
        except ValueError:
            pass
        else:
            email = email_name + '@' + domain_part.lower()
        return email

    def make_random_password(self, length=10,
                             allowed_chars='abcdefghjkmnpqrstuvwxyz'
                                           'ABCDEFGHJKLMNPQRSTUVWXYZ'
                                           '23456789'):
        """
        Generate a random password with the given length and given
        allowed_chars. The default value of allowed_chars does not have "I" or
        "O" or letters and digits that look similar -- just to avoid confusion.
        """
        return get_random_string(length, allowed_chars)

    def get_by_natural_key(self, username):
        return self.get(**{self.model.USERNAME_FIELD: username})



# Generated by Django 3.2.6 on 2021-08-23 12:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0112_auto_20210823_1246'),
    ]

    operations = [

    ]



# Generated by Django 2.2.5 on 2019-09-03 21:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0047_auto_20190903_2126'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='messages',
            name='checksum',
        ),
    ]



import logging

from isc_common.auth.models.user import User
from isc_common.fields.code_field import JSONFieldIVC
from isc_common.fields.related import ForeignKeyCascade
from isc_common.models.audit import AuditModel
from permission.models.widgets_trees import Widgets_trees

logger = logging.getLogger( __name__ )


class User_permission( AuditModel ) :
    user = ForeignKeyCascade( User )
    widget = ForeignKeyCascade( Widgets_trees )
    permission = JSONFieldIVC()

    def __str__( self ) :
        return f"{self.user.username}: {self.widget.id_widget}"

    class Meta :
        verbose_name = 'Доступы для пользователей'
        unique_together = (("user" , "widget") ,)



import logging

from django.db import transaction
from django.db.models import BigIntegerField

from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import BaseRefManager, BaseRefQuerySet, BaseRef
from isc_common.models.tree_audit import TreeAuditModelManager
from react.models.fragments_item_types import Fragments_item_types
from react.models.page_fragments import Page_fragments
from react.models.pages import Pages

logger = logging.getLogger(__name__)


class Fragments_4_page_viewQuerySet(BaseRefQuerySet):
    pass


class Fragments_4_page_viewManager(BaseRefManager):
    def deleteFromRequest(self, request, removed=None, ):
        request = DSRequest(request=request)
        res = 0
        tuple_ids = request.get_olds_tuple_ids('page_fragment_id')
        with transaction.atomic():
            for id, mode in tuple_ids:
                if mode == 'hide':
                    Page_fragments.objects.filter(id=id).soft_delete()
                    res += 1
                elif mode == 'visible':
                    Page_fragments.objects.filter(id=id).soft_restore()
                else:
                    qty, _ = Page_fragments.objects.filter(id=id).delete()
                    res += qty
        return res

    @classmethod
    def getRecord(cls, record ) :
        res = {
            'code': record.code,
            'deliting': record.deliting,
            'description': record.description,
            'editing': record.editing,
            'id': record.id,
            'name': record.name,
            'page_fragment_id': record.page_fragment_id,
            'parent_id': record.parent_id,
            'type__name': record.type.name,
            'type_id': record.type.id,
        }
        return res

    def get_queryset(self):
        return Fragments_4_page_viewQuerySet(self.model, using=self._db)


class Fragments_4_page_view(BaseRef):
    parent_id = BigIntegerField(db_index=True, null=True, blank=True)
    page = ForeignKeyProtect(Pages)
    type = ForeignKeyProtect(Fragments_item_types)
    page_fragment_id = BigIntegerField()

    objects = Fragments_4_page_viewManager()
    tree_objects = TreeAuditModelManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Структура фрагмента'
        db_table = 'react_fragments_4_page_view'
        managed = False
